@@ -129,20 +129,14 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_collection : public util::custom_allocable
 {
-	xmysqlnd_collection * collection;
+	~st_mysqlx_collection()
+	{
+		xmysqlnd_collection_free(collection, nullptr, nullptr);
+	}
+	xmysqlnd_collection* collection;
 };
 
 
-#define MYSQLX_FETCH_COLLECTION_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_collection*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->collection) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
-
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, __construct)
 {
 	UNUSED_INTERNAL_FUNCTION_PARAMETERS();
@@ -164,10 +158,9 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, getSession)
 		DBG_VOID_RETURN;
 	}
 
-	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
-
 	RETVAL_FALSE;
 
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
 	XMYSQLND_SESSION session{ data_object.collection->get_schema()->get_session()};
 	mysqlx_new_session(return_value, session);
 
@@ -176,23 +169,17 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, getSession)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, getName)
 {
-	st_mysqlx_collection* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_collection::getName");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_collection_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
 
-	if (object->collection) {
-		RETVAL_STRINGL(object->collection->get_name().s, object->collection->get_name().l);
-	} else {
-		RETVAL_FALSE;
-	}
+	RETVAL_STRINGL(data_object.collection->get_name().s, data_object.collection->get_name().l);
 
 	DBG_VOID_RETURN;
 }
@@ -216,29 +203,19 @@ mysqlx_collection_on_error(void * /*context*/, XMYSQLND_SESSION session,
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, existsInDatabase)
 {
-	st_mysqlx_collection* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_collection::existsInDatabase");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_collection_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	xmysqlnd_collection * collection = object->collection;
-	if (collection) {
-		const struct st_xmysqlnd_session_on_error_bind on_error = { mysqlx_collection_on_error, nullptr };
-		zval exists;
-		ZVAL_UNDEF(&exists);
-		if (PASS == collection->exists_in_database( on_error, &exists)) {
-			ZVAL_COPY_VALUE(return_value, &exists);
-		}
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
+	const st_xmysqlnd_session_on_error_bind on_error{ mysqlx_collection_on_error, nullptr };
+	data_object.collection->exists_in_database( on_error, return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -257,14 +234,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, count)
 	RETVAL_LONG(0);
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
-	xmysqlnd_collection* collection{ data_object.collection };
-	if (collection) {
-		const st_xmysqlnd_session_on_error_bind on_error{ mysqlx_collection_on_error, nullptr };
-		zval counter;
-		ZVAL_UNDEF(&counter);
-		if (PASS == collection->count(on_error, &counter)) {
-			ZVAL_COPY_VALUE(return_value, &counter);
-		}
+	const st_xmysqlnd_session_on_error_bind on_error{ mysqlx_collection_on_error, nullptr };
+	util::zvalue counter;
+	if (PASS == data_object.collection->count(on_error, counter.ptr())) {
+		counter.copy_to(return_value);
 	}
 
 	DBG_VOID_RETURN;
@@ -272,12 +245,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, count)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, getSchema)
 {
-	st_mysqlx_collection* object{nullptr};
-	XMYSQLND_SESSION session;
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_collection::getSchema");
-
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(
 				execute_data,
 				getThis(), "O",
@@ -286,18 +255,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, getSchema)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
 	RETVAL_FALSE;
 
-	if( object->collection &&
-		object->collection->get_schema() ) {
-		session = object->collection->get_schema()->get_session();
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
+	XMYSQLND_SESSION session;
+	auto& coll = data_object.collection;
+	if (coll->get_schema() ) {
+		session = coll->get_schema()->get_session();
 	}
 
 	if(session != nullptr) {
-		MYSQLND_STRING schema_name{ object->collection->get_schema()->get_name() };
-		xmysqlnd_schema * schema = session->create_schema_object(
-					mnd_str2c(schema_name));
+		MYSQLND_STRING schema_name{ coll->get_schema()->get_name() };
+		xmysqlnd_schema* schema = session->create_schema_object(mnd_str2c(schema_name));
 		if (schema) {
 			mysqlx_new_schema(return_value, schema);
 		} else {
@@ -313,13 +282,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, getSchema)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, add)
 {
-	st_mysqlx_collection* object{nullptr};
+	DBG_ENTER("mysqlx_collection::add");
 	zval* object_zv{nullptr};
 	zval* docs{nullptr};
 	int num_of_docs{0};
-
-	DBG_ENTER("mysqlx_collection::add");
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O+",
 												&object_zv,
 												mysqlx_collection_class_entry,
@@ -329,27 +295,21 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, add)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->collection) {
-		mysqlx_new_collection__add(return_value, object->collection,
-										docs,
-										num_of_docs);
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
+	mysqlx_new_collection__add(return_value, data_object.collection,
+									docs,
+									num_of_docs);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, find)
 {
-	st_mysqlx_collection* object{nullptr};
+	DBG_ENTER("mysqlx_collection::find");
 	zval* object_zv{nullptr};
 	util::string_view search_expr;
-
-	DBG_ENTER("mysqlx_collection::find");
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O|s",
 												&object_zv, mysqlx_collection_class_entry,
 												&(search_expr.str), &(search_expr.len)))
@@ -357,25 +317,19 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, find)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->collection) {
-		mysqlx_new_collection__find(return_value, search_expr, object->collection);
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
+	mysqlx_new_collection__find(return_value, search_expr, data_object.collection);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, modify)
 {
-	st_mysqlx_collection* object{nullptr};
+	DBG_ENTER("mysqlx_collection::modify");
 	zval* object_zv{nullptr};
 	util::string_view search_expr;
-
-	DBG_ENTER("mysqlx_collection::modify");
-
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data, getThis(), "Os",
 		&object_zv, mysqlx_collection_class_entry,
@@ -384,25 +338,19 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, modify)
 		throw util::xdevapi_exception(util::xdevapi_exception::Code::modify_fail);
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->collection) {
-		mysqlx_new_collection__modify(return_value, search_expr, object->collection);
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
+	mysqlx_new_collection__modify(return_value, search_expr, data_object.collection);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, remove)
 {
-	st_mysqlx_collection* object{nullptr};
+	DBG_ENTER("mysqlx_collection::remove");
 	zval* object_zv{nullptr};
 	util::string_view search_expr;
-
-	DBG_ENTER("mysqlx_collection::remove");
-
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data, getThis(), "Os",
 		&object_zv, mysqlx_collection_class_entry,
@@ -411,13 +359,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, remove)
 		throw util::xdevapi_exception(util::xdevapi_exception::Code::remove_fail);
 	}
 
-	MYSQLX_FETCH_COLLECTION_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->collection) {
-		mysqlx_new_collection__remove(return_value, search_expr, object->collection);
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_collection>(object_zv) };
+	mysqlx_new_collection__remove(return_value, search_expr, data_object.collection);
 
 	DBG_VOID_RETURN;
 }
@@ -485,8 +430,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, replaceOne)
 	}
 
 	const util::string_view Doc_root_path("$");
-	util::zvalue doc_with_id;
-	util::json::ensure_doc_id(doc, id, doc_with_id.ptr());
+	util::zvalue doc_with_id(util::json::ensure_doc_id(doc, id));
 	if (coll_modify.set(Doc_root_path, doc_with_id.ptr())) {
 		coll_modify.execute(return_value);
 	}
@@ -514,8 +458,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_collection, addOrReplaceOne)
 	auto& data_object = util::fetch_data_object<st_mysqlx_collection>(object_zv);
 
 	Collection_add coll_add;
-	util::zvalue doc_with_id;
-	util::json::ensure_doc_id(doc, id, doc_with_id.ptr());
+	util::zvalue doc_with_id(util::json::ensure_doc_id(doc, id));
 	if (coll_add.add_docs(data_object.collection, id, doc_with_id.ptr())) {
 		coll_add.execute(return_value);
 	}
@@ -627,7 +570,7 @@ static const zend_function_entry mysqlx_collection_methods[] = {
 };
 
 static zval *
-mysqlx_collection_property__name(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_collection_property__name(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_collection* object = (const st_mysqlx_collection* ) (obj->ptr);
 	DBG_ENTER("mysqlx_collection_property__name");
@@ -658,21 +601,11 @@ const struct st_mysqlx_property_entry mysqlx_collection_property_entries[] =
 static void
 mysqlx_collection_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_collection* inner_obj = (st_mysqlx_collection*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->collection) {
-			xmysqlnd_collection_free(inner_obj->collection, nullptr, nullptr);
-			inner_obj->collection = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_collection>(object);
 }
 
 static zend_object *
-php_mysqlx_collection_object_allocator(zend_class_entry * class_type)
+php_mysqlx_collection_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_collection_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_collection>(
@@ -683,23 +616,19 @@ php_mysqlx_collection_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_collection_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_collection_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_collection_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_collection_handlers.free_obj = mysqlx_collection_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Collection", mysqlx_collection_methods);
-		tmp_ce.create_object = php_mysqlx_collection_object_allocator;
-		mysqlx_collection_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_collection_class_entry, 1, mysqlx_schema_object_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_collection_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_collection_properties, mysqlx_collection_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_collection_class_entry,
+		"Collection",
+		mysqlx_std_object_handlers,
+		mysqlx_object_collection_handlers,
+		php_mysqlx_collection_object_allocator,
+		mysqlx_collection_free_storage,
+		mysqlx_collection_methods,
+		mysqlx_collection_properties,
+		mysqlx_collection_property_entries,
+		mysqlx_schema_object_interface_entry);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_collection_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -712,21 +641,13 @@ mysqlx_unregister_collection_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_collection(zval * return_value, xmysqlnd_collection * collection, const zend_bool clone)
+mysqlx_new_collection(zval* return_value, xmysqlnd_collection* collection)
 {
 	DBG_ENTER("mysqlx_new_collection");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_collection_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_collection* const object = (st_mysqlx_collection*) mysqlx_object->ptr;
-		if (object) {
-			object->collection = clone? collection->get_reference() : collection;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_collection& data_object{
+		util::init_object<st_mysqlx_collection>(mysqlx_collection_class_entry, return_value) };
+	data_object.collection = collection;
 
 	DBG_VOID_RETURN;
 }
