@@ -121,16 +121,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_column_result_is_padded,
 					   0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-#define MYSQLX_FETCH_COLUMN_RESULT_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_column_result*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
-} \
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_column_result, __construct)
 {
@@ -266,10 +256,8 @@ void
 get_column_meta_field(INTERNAL_FUNCTION_PARAMETERS,
 					meta_fields selected_meta_field)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_column_result* object{nullptr};
 	DBG_ENTER("get_column_meta_field");
-
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 								&object_zv,
 								mysqlx_column_result_class_entry))
@@ -277,49 +265,49 @@ get_column_meta_field(INTERNAL_FUNCTION_PARAMETERS,
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_COLUMN_RESULT_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->meta) {
+	auto& data_object{ util::fetch_data_object<st_mysqlx_column_result>(object_zv) };
+	const drv::st_xmysqlnd_result_field_meta* meta{ data_object.meta };
+	if (meta) {
 		switch(selected_meta_field) {
 		case schema_name:
-			ZVAL_STRINGL(return_value,object->meta->schema.s,
-						 object->meta->schema.l);
+			ZVAL_STRINGL(return_value,meta->schema.s,
+						 meta->schema.l);
 			break;
 		case table_name:
-			ZVAL_STRINGL(return_value,object->meta->original_table.s,
-						 object->meta->original_table.l);
+			ZVAL_STRINGL(return_value,meta->original_table.s,
+						 meta->original_table.l);
 			break;
 		case table_label:
-			ZVAL_STRINGL(return_value,object->meta->table.s,
-						 object->meta->table.l);
+			ZVAL_STRINGL(return_value,meta->table.s,
+						 meta->table.l);
 			break;
 		case column_name:
-			ZVAL_STRINGL(return_value,object->meta->original_name.s,
-						 object->meta->original_name.l);
+			ZVAL_STRINGL(return_value,meta->original_name.s,
+						 meta->original_name.l);
 			break;
 		case column_label:
-			ZVAL_STRINGL(return_value,object->meta->name.s,
-						 object->meta->name.l);
+			ZVAL_STRINGL(return_value,meta->name.s,
+						 meta->name.l);
 			break;
 		case type:
-			ZVAL_LONG(return_value,static_cast<zend_long>(get_column_type(object->meta)));
+			ZVAL_LONG(return_value,static_cast<zend_long>(get_column_type(meta)));
 			break;
 		case length:
-			get_column_length(return_value, object->meta->length);
+			get_column_length(return_value, meta->length);
 			break;
 		case fractional_digit:
-			ZVAL_LONG(return_value,object->meta->fractional_digits);
+			ZVAL_LONG(return_value,meta->fractional_digits);
 			break;
 		case is_number_signed:
-			ZVAL_BOOL(return_value,is_type_signed(object->meta));
+			ZVAL_BOOL(return_value,is_type_signed(meta));
 			break;
 		case collation_name:
 		case characterset_name:
 			{
 				const st_mysqlnd_charset* set =
-					mysqlnd_find_charset_nr(static_cast<unsigned int>(object->meta->collation));
+					mysqlnd_find_charset_nr(static_cast<unsigned int>(meta->collation));
 				if( set != nullptr && set->collation != nullptr ) {
 					if( selected_meta_field == collation_name ) {
 						ZVAL_STRINGL(return_value,
@@ -340,8 +328,8 @@ get_column_meta_field(INTERNAL_FUNCTION_PARAMETERS,
 		case is_padded:
 			{
 				zend_bool is_padded{FALSE};
-				if( object->meta->type == XMYSQLND_TYPE_BYTES &&
-					(object->meta->flags_set && object->meta->flags & BYTES_RIGHTPAD)) {
+				if( meta->type == XMYSQLND_TYPE_BYTES &&
+					(meta->flags_set && meta->flags & BYTES_RIGHTPAD)) {
 					is_padded = TRUE;
 				}
 				ZVAL_BOOL(return_value,is_padded);
@@ -351,8 +339,6 @@ get_column_meta_field(INTERNAL_FUNCTION_PARAMETERS,
 			RAISE_EXCEPTION(err_msg_meta_fail);
 			break;
 		}
-
-
 	} else {
 		RAISE_EXCEPTION(err_msg_meta_fail);
 	}
@@ -522,31 +508,20 @@ const struct st_mysqlx_property_entry mysqlx_column_result_property_entries[] =
 
 void
 mysqlx_new_column_result(
-	zval * return_value,
-	const st_xmysqlnd_result_field_meta * meta)
+	zval* return_value,
+	const st_xmysqlnd_result_field_meta* meta)
 {
 	DBG_ENTER("mysqlx_new_column");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_column_result_class_entry) &&
-			IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_column_result* const object =
-				(st_mysqlx_column_result*) mysqlx_object->ptr;
-		if (object) {
-			object->meta = meta;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s",
-							 ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_column_result& data_object{
+		util::init_object<st_mysqlx_column_result>(mysqlx_column_result_class_entry, return_value) };
+	data_object.meta = meta;
 
 	DBG_VOID_RETURN;
 }
 
 static zend_object *
-php_mysqlx_column_result_object_allocator(zend_class_entry * class_type)
+php_mysqlx_column_result_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_column_result_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_column_result>(
@@ -559,43 +534,23 @@ php_mysqlx_column_result_object_allocator(zend_class_entry * class_type)
 static void
 mysqlx_column_result_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_column_result* inner_obj = (
-				st_mysqlx_column_result* ) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		//Do not delete meta, that's someone else responsability
-		inner_obj->meta = nullptr;
-		mnd_efree(inner_obj);
-	}
-
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_column_result>(object);
 }
 
 void
 mysqlx_register_column_result_class(UNUSED_INIT_FUNC_ARGS,
-						zend_object_handlers * mysqlx_std_object_handlers)
+						zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_column_result_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_column_result_handlers.free_obj =
-			mysqlx_column_result_free_storage;
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce,
-					"mysql_xdevapi",
-					"ColumnResult",
-					mysqlx_column_result_methods);
-		tmp_ce.create_object = php_mysqlx_column_result_object_allocator;
-
-		mysqlx_column_result_class_entry = zend_register_internal_class(&tmp_ce);
-	}
-
-	zend_hash_init(&mysqlx_column_result_properties,
-				   0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_column_result_properties,
-					mysqlx_column_result_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_column_result_class_entry,
+		"ColumnResult",
+		mysqlx_std_object_handlers,
+		mysqlx_object_column_result_handlers,
+		php_mysqlx_column_result_object_allocator,
+		mysqlx_column_result_free_storage,
+		mysqlx_column_result_methods,
+		mysqlx_column_result_properties,
+		mysqlx_column_result_property_entries);
 }
 
 void
