@@ -58,17 +58,10 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_doc_result__get_warnings, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-#define MYSQLX_FETCH_DOC_RESULT_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_doc_result*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
-} \
-
+st_mysqlx_doc_result::~st_mysqlx_doc_result()
+{
+	xmysqlnd_stmt_result_free(result, nullptr, nullptr);
+}
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, __construct)
 {
@@ -77,28 +70,23 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, fetchOne)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_doc_result* object{nullptr};
-
-	RETVAL_NULL();
-
 	DBG_ENTER("mysqlx_doc_result::fetchOne");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_doc_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_DOC_RESULT_FROM_ZVAL(object, object_zv);
+	RETVAL_NULL();
 
-	if (object && object->result && FALSE == object->result->m.eof(object->result)) {
-		zval row;
-		ZVAL_UNDEF(&row);
-		if (PASS == object->result->m.fetch_current(object->result, &row, nullptr, nullptr)) {
-			xmysqlnd_utils_decode_doc_row(&row, return_value);
-			//ZVAL_COPY_VALUE(return_value, &row);
-			zval_ptr_dtor(&row);
-			object->result->m.next(object->result, nullptr, nullptr);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_doc_result>(object_zv) };
+	drv::st_xmysqlnd_stmt_result* result{ data_object.result};
+	if (FALSE == data_object.result->m.eof(result)) {
+		util::zvalue row;
+		if (PASS == data_object.result->m.fetch_current(result, row.ptr(), nullptr, nullptr)) {
+			xmysqlnd_utils_decode_doc_row(row.ptr(), return_value);
+			data_object.result->m.next(result, nullptr, nullptr);
 		}
 	}
 	DBG_VOID_RETURN;
@@ -118,12 +106,9 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, fetchAll)
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_doc_result>(object_zv) };
 	if (data_object.result) {
-		zval set;
-		ZVAL_UNDEF(&set);
-		if (PASS == data_object.result->m.fetch_all(data_object.result, &set, nullptr, nullptr)) {
-			xmysqlnd_utils_decode_doc_rows(&set, return_value);
-			//ZVAL_COPY_VALUE(return_value, &set);
-			zval_ptr_dtor(&set);
+		util::zvalue set;
+		if (PASS == data_object.result->m.fetch_all(data_object.result, set.ptr(), nullptr, nullptr)) {
+			xmysqlnd_utils_decode_doc_rows(set.ptr(), return_value);
 		}
 	}
 	util::zend::ensure_is_array(return_value);
@@ -133,25 +118,21 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, fetchAll)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, getWarningsCount)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_doc_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_doc_result::getWarningsCount");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_doc_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_DOC_RESULT_FROM_ZVAL(object, object_zv);
-
 	RETVAL_LONG(0);
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_doc_result>(object_zv) };
 	size_t warning_count{ 0 };
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			warning_count = warnings->m->count(warnings);
-		}
+	const XMYSQLND_WARNING_LIST* const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		warning_count = warnings->m->count(warnings);
 	}
 	if (UNEXPECTED(warning_count >= ZEND_LONG_MAX)) {
 		ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(warning_count).c_str()));
@@ -165,33 +146,29 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, getWarningsCount)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_doc_result, getWarnings)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_doc_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_doc_result::getWarnings");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_doc_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_DOC_RESULT_FROM_ZVAL(object, object_zv);
 
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const unsigned int count{warnings->m->count(warnings)};
-			array_init_size(return_value, count);
-			for (unsigned int i{0}; i < count; ++i) {
-				const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
-				zval warning_zv;
+	auto& data_object{ util::fetch_data_object<st_mysqlx_doc_result>(object_zv) };
 
-				ZVAL_UNDEF(&warning_zv);
-				mysqlx_new_warning(&warning_zv, warning.message, warning.level, warning.code);
+	const XMYSQLND_WARNING_LIST * const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const unsigned int count{warnings->m->count(warnings)};
+		array_init_size(return_value, count);
+		for (unsigned int i{0}; i < count; ++i) {
+			const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
+			util::zvalue warning_zv;
+			mysqlx_new_warning(warning_zv.ptr(), warning.message, warning.level, warning.code);
 
-				if (Z_TYPE(warning_zv) != IS_UNDEF) {
-					zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &warning_zv);
-				}
+			if (!warning_zv.is_undef()) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), warning_zv.ptr());
+				warning_zv.invalidate();
 			}
 		}
 	}
@@ -223,20 +200,11 @@ const struct st_mysqlx_property_entry mysqlx_doc_result_property_entries[] =
 static void
 mysqlx_doc_result_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_doc_result* inner_obj = (st_mysqlx_doc_result*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->result) {
-			xmysqlnd_stmt_result_free(inner_obj->result, nullptr, nullptr);
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_doc_result>(object);
 }
 
 static zend_object *
-php_mysqlx_doc_result_object_allocator(zend_class_entry * class_type)
+php_mysqlx_doc_result_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_doc_result_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_doc_result>(
@@ -247,25 +215,21 @@ php_mysqlx_doc_result_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_doc_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_doc_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_doc_result_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_doc_result_handlers.free_obj = mysqlx_doc_result_free_storage;
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "DocResult", mysqlx_doc_result_methods);
-		tmp_ce.create_object = php_mysqlx_doc_result_object_allocator;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_doc_result_class_entry,
+		"DocResult",
+		mysqlx_std_object_handlers,
+		mysqlx_object_doc_result_handlers,
+		php_mysqlx_doc_result_object_allocator,
+		mysqlx_doc_result_free_storage,
+		mysqlx_doc_result_methods,
+		mysqlx_doc_result_properties,
+		mysqlx_doc_result_property_entries,
+		mysqlx_base_result_interface_entry);
 
-		mysqlx_doc_result_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_doc_result_class_entry, 1, mysqlx_base_result_interface_entry);
-
-		mysqlx_register_doc_result_iterator(mysqlx_doc_result_class_entry);
-	}
-
-	zend_hash_init(&mysqlx_doc_result_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_doc_result_properties, mysqlx_doc_result_property_entries);
+	mysqlx_register_doc_result_iterator(mysqlx_doc_result_class_entry);
 }
 
 void
@@ -275,21 +239,13 @@ mysqlx_unregister_doc_result_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_doc_result(zval * return_value, XMYSQLND_STMT_RESULT * result)
+mysqlx_new_doc_result(zval* return_value, XMYSQLND_STMT_RESULT* result)
 {
 	DBG_ENTER("mysqlx_new_doc_result");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_doc_result_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_doc_result* const object = (st_mysqlx_doc_result*) mysqlx_object->ptr;
-		if (object) {
-			object->result = result;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_doc_result& data_object{
+		util::init_object<st_mysqlx_doc_result>(mysqlx_doc_result_class_entry, return_value) };
+	data_object.result = result;
 
 	DBG_VOID_RETURN;
 }
