@@ -26,6 +26,7 @@ extern "C" {
 #include "mysqlx_class_properties.h"
 
 #include "mysqlx_exception.h"
+#include "util/object.h"
 
 namespace mysqlx {
 
@@ -49,22 +50,12 @@ RAISE_EXCEPTION(const int errcode, const char * const msg)
 	mysqlx_new_exception(errcode, sqlstate, errmsg);
 }
 
-#define MYSQLX_FETCH_EXCEPTION_FROM_ZO(_to, _from) \
-{ \
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo((_from)); \
-	(_to) = (st_mysqlx_exception*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-}
-
 static const zend_function_entry mysqlx_exception_methods[] = {
 	{nullptr, nullptr, nullptr}
 };
 
 static zval *
-mysqlx_exception_property__message(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_exception_property__message(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_exception* object = (const st_mysqlx_exception* ) (obj->ptr);
 	DBG_ENTER("mysqlx_exception_property__message");
@@ -84,7 +75,7 @@ mysqlx_exception_property__message(const st_mysqlx_object* obj, zval * return_va
 }
 
 static zval *
-mysqlx_exception_property__level(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_exception_property__level(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_exception* object = (const st_mysqlx_exception* ) (obj->ptr);
 	DBG_ENTER("mysqlx_exception_property__level");
@@ -93,7 +84,7 @@ mysqlx_exception_property__level(const st_mysqlx_object* obj, zval * return_valu
 }
 
 static zval *
-mysqlx_exception_property__code(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_exception_property__code(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_exception* object = (const st_mysqlx_exception* ) (obj->ptr);
 	DBG_ENTER("mysqlx_exception_property__code");
@@ -115,19 +106,17 @@ static HashTable mysqlx_exception_properties;
 
 
 void
-mysqlx_register_exception_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_exception_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_exception_handlers = *mysqlx_std_object_handlers;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Exception", mysqlx_exception_methods);
-		mysqlx_exception_class_entry = zend_register_internal_class_ex(&tmp_ce, spl_ce_RuntimeException);
-	}
-
-	zend_hash_init(&mysqlx_exception_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	mysqlx_add_properties(&mysqlx_exception_properties, mysqlx_exception_property_entries);
+	MYSQL_XDEVAPI_REGISTER_DERIVED_CLASS(
+		mysqlx_exception_class_entry,
+		spl_ce_RuntimeException,
+		"Exception",
+		mysqlx_std_object_handlers,
+		mysqlx_object_exception_handlers,
+		mysqlx_exception_methods,
+		mysqlx_exception_properties,
+		mysqlx_exception_property_entries);
 }
 
 void
