@@ -46,18 +46,6 @@ struct st_mysqlx_execution_status : public util::custom_allocable
 #define TYPE_NAME_ENABLED 1
 
 
-#define MYSQLX_FETCH__EXECUTION_STATUS_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_execution_status*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
-} \
-
-
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_execution_status, __construct)
 {
 	UNUSED_INTERNAL_FUNCTION_PARAMETERS();
@@ -69,7 +57,7 @@ static const zend_function_entry mysqlx_execution_status_methods[] = {
 };
 
 static zval *
-mysqlx_execution_status_property__affected_items(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_execution_status_property__affected_items(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_execution_status* object = (st_mysqlx_execution_status*)(obj->ptr);
 	DBG_ENTER("mysqlx_execution_status_property__affected_items");
@@ -78,7 +66,7 @@ mysqlx_execution_status_property__affected_items(const st_mysqlx_object* obj, zv
 }
 
 static zval *
-mysqlx_execution_status_property__matched_items(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_execution_status_property__matched_items(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_execution_status* object = (st_mysqlx_execution_status*)(obj->ptr);
 	DBG_ENTER("mysqlx_execution_status_property__matched_items");
@@ -87,7 +75,7 @@ mysqlx_execution_status_property__matched_items(const st_mysqlx_object* obj, zva
 }
 
 static zval *
-mysqlx_execution_status_property__found_items(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_execution_status_property__found_items(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_execution_status* object = (st_mysqlx_execution_status*)(obj->ptr);
 	DBG_ENTER("mysqlx_execution_status_property__found_items");
@@ -96,7 +84,7 @@ mysqlx_execution_status_property__found_items(const st_mysqlx_object* obj, zval
 }
 
 static zval *
-mysqlx_execution_status_property__last_insert_id(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_execution_status_property__last_insert_id(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_execution_status* object = (st_mysqlx_execution_status*)(obj->ptr);
 	DBG_ENTER("mysqlx_execution_status_property__last_insert_id");
@@ -105,7 +93,7 @@ mysqlx_execution_status_property__last_insert_id(const st_mysqlx_object* obj, zv
 }
 
 static zval *
-mysqlx_execution_status_property__last_document_id(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_execution_status_property__last_document_id(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_execution_status* object = (st_mysqlx_execution_status*)(obj->ptr);
 	DBG_ENTER("mysqlx_execution_status_property__last_document_id");
@@ -129,17 +117,11 @@ static HashTable mysqlx_execution_status_properties;
 static void
 mysqlx_execution_status_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_execution_status* message = (st_mysqlx_execution_status*) mysqlx_object->ptr;
-
-	if (message) {
-		mnd_efree(message);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_execution_status>(object);
 }
 
 static zend_object *
-php_mysqlx_execution_status_object_allocator(zend_class_entry * class_type)
+php_mysqlx_execution_status_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_execution_status_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_execution_status>(
@@ -150,21 +132,18 @@ php_mysqlx_execution_status_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_execution_status_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_execution_status_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_execution_status_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_execution_status_handlers.free_obj = mysqlx_execution_status_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "ExecutionStatus", mysqlx_execution_status_methods);
-		tmp_ce.create_object = php_mysqlx_execution_status_object_allocator;
-		mysqlx_execution_status_class_entry = zend_register_internal_class(&tmp_ce);
-	}
-
-	zend_hash_init(&mysqlx_execution_status_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	mysqlx_add_properties(&mysqlx_execution_status_properties, mysqlx_execution_status_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_execution_status_class_entry,
+		"ExecutionStatus",
+		mysqlx_std_object_handlers,
+		mysqlx_object_execution_status_handlers,
+		php_mysqlx_execution_status_object_allocator,
+		mysqlx_execution_status_free_storage,
+		mysqlx_execution_status_methods,
+		mysqlx_execution_status_properties,
+		mysqlx_execution_status_property_entries);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_execution_status_class_entry, "affectedItems",	sizeof("affectedItems") - 1,	ZEND_ACC_PUBLIC);
@@ -181,24 +160,16 @@ mysqlx_unregister_execution_status_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_execution_status(zval * return_value, const XMYSQLND_STMT_EXECUTION_STATE * const status)
+mysqlx_new_execution_status(zval* return_value, const XMYSQLND_STMT_EXECUTION_STATE* const status)
 {
 	DBG_ENTER("mysqlx_new_execution_status");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_execution_status_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_execution_status* const object = (st_mysqlx_execution_status*) mysqlx_object->ptr;
-		if (object) {
-			object->items_affected = status->m->get_affected_items_count(status);
-			object->items_matched = status->m->get_matched_items_count(status);
-			object->items_found = status->m->get_found_items_count(status);
-			object->last_insert_id = status->m->get_last_insert_id(status);
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_execution_status& data_object{
+		util::init_object<st_mysqlx_execution_status>(mysqlx_execution_status_class_entry, return_value) };
+	data_object.items_affected = status->m->get_affected_items_count(status);
+	data_object.items_matched = status->m->get_matched_items_count(status);
+	data_object.items_found = status->m->get_found_items_count(status);
+	data_object.last_insert_id = status->m->get_last_insert_id(status);
 
 	DBG_VOID_RETURN;
 }
