@@ -36,6 +36,10 @@ static zend_class_entry * mysqlx_expression_class_entry;
 
 struct st_mysqlx_expression : public util::custom_allocable
 {
+	~st_mysqlx_expression()
+	{
+		zval_ptr_dtor(&expression);
+	}
 	zval expression;
 };
 
@@ -44,18 +48,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_expression__construct, 0, ZEND_RETURN_VALU
 ZEND_END_ARG_INFO()
 
 
-#define MYSQLX_FETCH_EXPRESSION_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_expression*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->expr) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
-
-
-
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_expression, __construct)
 {
 	UNUSED(return_value);
@@ -116,20 +108,13 @@ const struct st_mysqlx_property_entry mysqlx_expression_property_entries[] =
 };
 
 static void
-mysqlx_expression_free_storage(zend_object * object)
+mysqlx_expression_free_storage(zend_object* object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_expression* inner_obj = (st_mysqlx_expression*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		zval_ptr_dtor(&inner_obj->expression);
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_expression>(object);
 }
 
 static zend_object *
-php_mysqlx_expression_object_allocator(zend_class_entry * class_type)
+php_mysqlx_expression_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_expression_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_expression>(
@@ -140,22 +125,18 @@ php_mysqlx_expression_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_expression_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_expression_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_expression_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_expression_handlers.free_obj = mysqlx_expression_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Expression", mysqlx_expression_methods);
-		tmp_ce.create_object = php_mysqlx_expression_object_allocator;
-		mysqlx_expression_class_entry = zend_register_internal_class(&tmp_ce);
-	}
-
-	zend_hash_init(&mysqlx_expression_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_expression_properties, mysqlx_expression_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_expression_class_entry,
+		"Expression",
+		mysqlx_std_object_handlers,
+		mysqlx_object_expression_handlers,
+		php_mysqlx_expression_object_allocator,
+		mysqlx_expression_free_storage,
+		mysqlx_expression_methods,
+		mysqlx_expression_properties,
+		mysqlx_expression_property_entries);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_expression_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -168,22 +149,13 @@ mysqlx_unregister_expression_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_expression(zval * return_value, const MYSQLND_CSTRING expression)
+mysqlx_new_expression(zval* return_value, const MYSQLND_CSTRING expression)
 {
 	DBG_ENTER("mysqlx_new_expression");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_expression_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_expression* const object = (st_mysqlx_expression*) mysqlx_object->ptr;
-		if (object) {
-			DBG_INF_FMT("expression=[%*s]", expression.l, expression.s);
-			ZVAL_STRINGL(&object->expression, expression.s, expression.l);
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_expression& data_object{
+		util::init_object<st_mysqlx_expression>(mysqlx_expression_class_entry, return_value) };
+	ZVAL_STRINGL(&data_object.expression, expression.s, expression.l);
 
 	DBG_VOID_RETURN;
 }
