@@ -59,17 +59,10 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_result__get_warnings, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-#define MYSQLX_FETCH_RESULT_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_result*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
-} \
-
+st_mysqlx_result::~st_mysqlx_result()
+{
+	xmysqlnd_stmt_result_free(result, nullptr, nullptr);
+}
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, __construct)
 {
@@ -78,30 +71,28 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAffectedItemsCount)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_result::getAffectedItemsCount");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_RESULT_FROM_ZVAL(object, object_zv);
 
 	RETVAL_LONG(0);
-	if (object->result) {
-		const XMYSQLND_STMT_EXECUTION_STATE * exec_state = object->result->exec_state;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (exec_state) {
-			const size_t value = exec_state->m->get_affected_items_count(exec_state);
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				RETVAL_LONG(value);
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_result>(object_zv) };
+
+	const XMYSQLND_STMT_EXECUTION_STATE* exec_state = data_object.result->exec_state;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (exec_state) {
+		const size_t value = exec_state->m->get_affected_items_count(exec_state);
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			RETVAL_LONG(value);
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -109,31 +100,29 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAffectedItemsCount)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getAutoIncrementValue)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_result::getAutoIncrementValue");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_RESULT_FROM_ZVAL(object, object_zv);
 
 	RETVAL_FALSE;
-	if (object->result && object->result->exec_state) {
-		const XMYSQLND_STMT_EXECUTION_STATE * const exec_state = object->result->exec_state;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (exec_state) {
-			const uint64_t value = exec_state->m->get_last_insert_id(exec_state);
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				const auto& value_str{ util::to_string(value) };
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", value_str.c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				ZVAL_LONG(return_value, static_cast<zend_long>(value));
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_result>(object_zv) };
+
+	const XMYSQLND_STMT_EXECUTION_STATE* const exec_state = data_object.result->exec_state;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (exec_state) {
+		const uint64_t value = exec_state->m->get_last_insert_id(exec_state);
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			const auto& value_str{ util::to_string(value) };
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", value_str.c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			ZVAL_LONG(return_value, static_cast<zend_long>(value));
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -143,63 +132,51 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getGeneratedIds)
 {
     DBG_ENTER("mysqlx_result::getGeneratedIds");
 	zval* object_zv{nullptr};
-	st_mysqlx_result* object{nullptr};
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_RESULT_FROM_ZVAL(object, object_zv);
 
-	if (object->result && object->result->exec_state) {
-		const XMYSQLND_STMT_EXECUTION_STATE * const exec_state = object->result->exec_state;
-		if ( exec_state == nullptr ) {
-			php_error_docref(nullptr, E_WARNING, "Unable to get the correct exec_state");
-			DBG_VOID_RETURN;
+	auto& data_object{ util::fetch_data_object<st_mysqlx_result>(object_zv) };
+	const XMYSQLND_STMT_EXECUTION_STATE* const exec_state = data_object.result->exec_state;
+	if ( exec_state == nullptr ) {
+		php_error_docref(nullptr, E_WARNING, "Unable to get the correct exec_state");
+		DBG_VOID_RETURN;
 
-		}
-		auto& ids = exec_state->generated_doc_ids;
-		const size_t num_of_docs = ids.size();
-		array_init_size(return_value, static_cast<uint32_t>(num_of_docs));
-		for( auto& elem : ids ) {
-			zval id;
-			ZVAL_STRINGL(&id,elem.c_str(),elem.size());
-			zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &id);
-		}
 	}
-
-	util::zend::ensure_is_array(return_value);
+	auto& ids = exec_state->generated_doc_ids;
+	util::zvalue docs(util::zvalue::create_array(ids.size()));
+	for( auto& elem : ids ) {
+		docs.push_back(elem);
+	}
+	docs.move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarningsCount)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_result::getWarningsCount");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_RESULT_FROM_ZVAL(object, object_zv);
-
 	RETVAL_LONG(0);
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const size_t value = warnings->m->count(warnings);
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				RETVAL_LONG(value);
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_result>(object_zv) };
+	const XMYSQLND_WARNING_LIST* const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const size_t value = warnings->m->count(warnings);
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			RETVAL_LONG(value);
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -207,33 +184,27 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarningsCount)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_result, getWarnings)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_result::getWarnings");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_RESULT_FROM_ZVAL(object, object_zv);
 
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const unsigned int count{warnings->m->count(warnings)};
-			array_init_size(return_value, count);
-			for (unsigned int i{0}; i < count; ++i) {
-				const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
-				zval warning_zv;
-
-				ZVAL_UNDEF(&warning_zv);
-				mysqlx_new_warning(&warning_zv, warning.message, warning.level, warning.code);
-
-				if (Z_TYPE(warning_zv) != IS_UNDEF) {
-					zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &warning_zv);
-				}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_result>(object_zv) };
+	const XMYSQLND_WARNING_LIST* const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const unsigned int count{warnings->m->count(warnings)};
+		array_init_size(return_value, count);
+		for (unsigned int i{0}; i < count; ++i) {
+			const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
+			util::zvalue warning_zv;
+			mysqlx_new_warning(warning_zv.ptr(), warning.message, warning.level, warning.code);
+			if (!warning_zv.is_undef()) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), warning_zv.ptr());
+				warning_zv.invalidate();
 			}
 		}
 	}
@@ -265,20 +236,11 @@ const struct st_mysqlx_property_entry mysqlx_result_property_entries[] =
 static void
 mysqlx_result_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_result* inner_obj = (st_mysqlx_result*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->result) {
-			xmysqlnd_stmt_result_free(inner_obj->result, nullptr, nullptr);
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_result>(object);
 }
 
 static zend_object *
-php_mysqlx_result_object_allocator(zend_class_entry * class_type)
+php_mysqlx_result_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_result_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_result>(
@@ -289,25 +251,21 @@ php_mysqlx_result_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_result_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_result_handlers.free_obj = mysqlx_result_free_storage;
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Result", mysqlx_result_methods);
-		tmp_ce.create_object = php_mysqlx_result_object_allocator;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_result_class_entry,
+		"Result",
+		mysqlx_std_object_handlers,
+		mysqlx_object_result_handlers,
+		php_mysqlx_result_object_allocator,
+		mysqlx_result_free_storage,
+		mysqlx_result_methods,
+		mysqlx_result_properties,
+		mysqlx_result_property_entries,
+		mysqlx_base_result_interface_entry);
 
-		mysqlx_result_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_result_class_entry, 1, mysqlx_base_result_interface_entry);
-
-		mysqlx_register_result_iterator(mysqlx_result_class_entry);
-	}
-
-	zend_hash_init(&mysqlx_result_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_result_properties, mysqlx_result_property_entries);
+	mysqlx_register_result_iterator(mysqlx_result_class_entry);
 }
 
 void
@@ -317,21 +275,13 @@ mysqlx_unregister_result_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_result(zval * return_value, XMYSQLND_STMT_RESULT * result)
+mysqlx_new_result(zval* return_value, XMYSQLND_STMT_RESULT* result)
 {
 	DBG_ENTER("mysqlx_new_result");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_result_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_result* const object = (st_mysqlx_result*) mysqlx_object->ptr;
-		if (object) {
-			object->result = result;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_result& data_object{
+		util::init_object<st_mysqlx_result>(mysqlx_result_class_entry, return_value) };
+	data_object.result = result;
 
 	DBG_VOID_RETURN;
 }
