@@ -70,16 +70,10 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_row_result__get_columns, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-#define MYSQLX_FETCH_ROW_RESULT_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_row_result*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
-} \
+st_mysqlx_row_result::~st_mysqlx_row_result()
+{
+	xmysqlnd_stmt_result_free(result, nullptr, nullptr);
+}
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, __construct)
 {
@@ -88,26 +82,22 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, fetchOne)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_row_result* object{nullptr};
-
-	RETVAL_NULL();
-
 	DBG_ENTER("mysqlx_row_result::fetchOne");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_row_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_ROW_RESULT_FROM_ZVAL(object, object_zv);
+	RETVAL_NULL();
 
-	if (object && object->result && FALSE == object->result->m.eof(object->result)) {
-		zval row;
-		ZVAL_UNDEF(&row);
-		if (PASS == object->result->m.fetch_current(object->result, &row, nullptr, nullptr)) {
-			ZVAL_COPY_VALUE(return_value, &row);
-			object->result->m.next(object->result, nullptr, nullptr);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_row_result>(object_zv) };
+	auto& obj_result{ data_object.result };
+	if (FALSE == obj_result->m.eof(obj_result)) {
+		if (PASS == obj_result->m.fetch_current(obj_result, return_value, nullptr, nullptr)) {
+			obj_result->m.next(obj_result, nullptr, nullptr);
 		}
 	}
 	DBG_VOID_RETURN;
@@ -125,13 +115,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, fetchAll)
 	}
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_row_result>(object_zv) };
-	if (data_object.result) {
-		zval set;
-		ZVAL_UNDEF(&set);
-		if (PASS == data_object.result->m.fetch_all(data_object.result, &set, nullptr, nullptr)) {
-			ZVAL_COPY_VALUE(return_value, &set);
-		}
-	}
+	data_object.result->m.fetch_all(data_object.result, return_value, nullptr, nullptr);
 	util::zend::ensure_is_array(return_value);
 
 	DBG_VOID_RETURN;
@@ -139,30 +123,27 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, fetchAll)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, getWarningsCount)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_row_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_row_result::getWarningsCount");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_row_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_ROW_RESULT_FROM_ZVAL(object, object_zv);
-
 	RETVAL_LONG(0);
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const size_t value = warnings->m->count(warnings);
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				RETVAL_LONG(value);
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_row_result>(object_zv) };
+	const XMYSQLND_WARNING_LIST* const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const size_t value = warnings->m->count(warnings);
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			RETVAL_LONG(value);
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -170,33 +151,28 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, getWarningsCount)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, getWarnings)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_row_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_row_result::getWarnings");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_row_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_ROW_RESULT_FROM_ZVAL(object, object_zv);
 
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const unsigned int count{warnings->m->count(warnings)};
-			array_init_size(return_value, count);
-			for (unsigned int i{0}; i < count; ++i) {
-				const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
-				zval warning_zv;
-
-				ZVAL_UNDEF(&warning_zv);
-				mysqlx_new_warning(&warning_zv, warning.message, warning.level, warning.code);
-
-				if (Z_TYPE(warning_zv) != IS_UNDEF) {
-					zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &warning_zv);
-				}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_row_result>(object_zv) };
+	const XMYSQLND_WARNING_LIST* const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const unsigned int count{warnings->m->count(warnings)};
+		array_init_size(return_value, count);
+		for (unsigned int i{0}; i < count; ++i) {
+			const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
+			util::zvalue warning_zv;
+			mysqlx_new_warning(warning_zv.ptr(), warning.message, warning.level, warning.code);
+			if (!warning_zv.is_undef()) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), warning_zv.ptr());
+				warning_zv.invalidate();
 			}
 		}
 	}
@@ -218,28 +194,18 @@ static st_xmysqlnd_stmt_result_meta* get_stmt_result_meta(st_xmysqlnd_stmt_resul
 
 static st_xmysqlnd_stmt_result_meta* get_stmt_result_meta(INTERNAL_FUNCTION_PARAMETERS)
 {
-	st_xmysqlnd_stmt_result_meta* meta{nullptr};
-	zval* object_zv{nullptr};
-	st_mysqlx_row_result* object{nullptr};
-
 	DBG_ENTER("get_stmt_result_meta");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_row_result_class_entry))
 	{
 		DBG_RETURN(nullptr);
 	}
 
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(object_zv);
-	object = (st_mysqlx_row_result*) mysqlx_object->ptr;
-	if (!object) {
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-		DBG_RETURN(nullptr);
-	}
-
+	auto& data_object{ util::fetch_data_object<st_mysqlx_row_result>(object_zv) };
 	RETVAL_FALSE;
-	if (object->result) {
-		meta = get_stmt_result_meta(object->result);
-	}
+	st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
 
 	if(meta == nullptr) {
 		RAISE_EXCEPTION(10001,"get_stmt_result_meta: Unable to extract metadata");
@@ -280,13 +246,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_row_result, getColumns)
 		array_init_size(return_value, count);
 		for (unsigned int i{0}; i < count; ++i) {
 			const XMYSQLND_RESULT_FIELD_META* column = meta->m->get_field(meta, i);
-			zval column_zv;
+			util::zvalue column_zv;
+			mysqlx_new_column_result(column_zv.ptr(), column);
 
-			ZVAL_UNDEF(&column_zv);
-			mysqlx_new_column_result(&column_zv, column);
-
-			if (Z_TYPE(column_zv) != IS_UNDEF) {
-				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &column_zv);
+			if (!column_zv.is_undef()) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), column_zv.ptr());
+				column_zv.invalidate();
 			}
 		}
 	}
@@ -346,22 +311,13 @@ const struct st_mysqlx_property_entry mysqlx_row_result_property_entries[] =
 };
 
 static void
-mysqlx_row_result_free_storage(zend_object * object)
+mysqlx_row_result_free_storage(zend_object* object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_row_result* inner_obj = (st_mysqlx_row_result*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->result) {
-			xmysqlnd_stmt_result_free(inner_obj->result, nullptr, nullptr);
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_row_result>(object);
 }
 
 static zend_object *
-php_mysqlx_row_result_object_allocator(zend_class_entry * class_type)
+php_mysqlx_row_result_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_row_result_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_row_result>(
@@ -372,25 +328,21 @@ php_mysqlx_row_result_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_row_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_row_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_row_result_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_row_result_handlers.free_obj = mysqlx_row_result_free_storage;
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "RowResult", mysqlx_row_result_methods);
-		tmp_ce.create_object = php_mysqlx_row_result_object_allocator;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_row_result_class_entry,
+		"RowResult",
+		mysqlx_std_object_handlers,
+		mysqlx_object_row_result_handlers,
+		php_mysqlx_row_result_object_allocator,
+		mysqlx_row_result_free_storage,
+		mysqlx_row_result_methods,
+		mysqlx_row_result_properties,
+		mysqlx_row_result_property_entries,
+		mysqlx_base_result_interface_entry);
 
-		mysqlx_row_result_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_row_result_class_entry, 1, mysqlx_base_result_interface_entry);
-
-		mysqlx_register_row_result_iterator(mysqlx_row_result_class_entry);
-	}
-
-	zend_hash_init(&mysqlx_row_result_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_row_result_properties, mysqlx_row_result_property_entries);
+	mysqlx_register_row_result_iterator(mysqlx_row_result_class_entry);
 }
 
 void
@@ -400,21 +352,13 @@ mysqlx_unregister_row_result_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_row_result(zval * return_value, XMYSQLND_STMT_RESULT * result)
+mysqlx_new_row_result(zval* return_value, XMYSQLND_STMT_RESULT* result)
 {
 	DBG_ENTER("mysqlx_new_row_result");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_row_result_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_row_result* const object = (st_mysqlx_row_result*) mysqlx_object->ptr;
-		if (object) {
-			object->result = result;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_row_result& data_object{
+		util::init_object<st_mysqlx_row_result>(mysqlx_row_result_class_entry, return_value) };
+	data_object.result = result;
 
 	DBG_VOID_RETURN;
 }
