@@ -64,6 +64,13 @@ ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_schema__create_collection, 0, ZEND_RETURN_VALUE, 1)
 	ZEND_ARG_TYPE_INFO(no_pass_by_ref, name, IS_STRING, dont_allow_null)
+	ZEND_ARG_TYPE_INFO(no_pass_by_ref, options, IS_STRING, dont_allow_null)
+ZEND_END_ARG_INFO()
+
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_schema_modify_collection, 0, ZEND_RETURN_VALUE, 2)
+	ZEND_ARG_TYPE_INFO(no_pass_by_ref, name, IS_STRING, dont_allow_null)
+	ZEND_ARG_TYPE_INFO(no_pass_by_ref, options, IS_STRING, dont_allow_null)
 ZEND_END_ARG_INFO()
 
 
@@ -97,19 +104,18 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_schema : public util::custom_allocable
 {
+	~st_mysqlx_schema();
 	xmysqlnd_schema* schema;
 };
 
+st_mysqlx_schema::~st_mysqlx_schema()
+{
+	xmysqlnd_schema_free(schema, nullptr, nullptr);
+	if (schema->get_counter() == 0) {
+		mnd_efree(schema);
+	}
+}
 
-#define MYSQLX_FETCH_SCHEMA_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_schema*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->schema) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, __construct)
 {
@@ -133,9 +139,6 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getSession)
 	}
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
-
-	RETVAL_FALSE;
-
 	XMYSQLND_SESSION session{ data_object.schema->get_session() };
 	mysqlx_new_session(return_value, session);
 
@@ -144,24 +147,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getSession)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getName)
 {
-	st_mysqlx_schema* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_schema::getName");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_schema_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-
-	RETVAL_FALSE;
-
-	if (object->schema) {
-		RETVAL_STRINGL(object->schema->get_name().s,
-				object->schema->get_name().l);
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	const auto& schema_name{ data_object.schema->get_name() };
+	RETVAL_STRINGL(schema_name.s, schema_name.l);
 
 	DBG_VOID_RETURN;
 }
@@ -182,29 +179,19 @@ mysqlx_scheme_on_error(
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, existsInDatabase)
 {
-	st_mysqlx_schema* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_schema::existsInDatabase");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_schema_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-
-	RETVAL_FALSE;
-
-	xmysqlnd_schema* schema = object->schema;
-	if (schema) {
-		const struct st_xmysqlnd_session_on_error_bind on_error = { mysqlx_scheme_on_error, nullptr };
-		zval exists;
-		ZVAL_UNDEF(&exists);
-		if (PASS == schema->exists_in_database(on_error, &exists)) {
-			ZVAL_COPY_VALUE(return_value, &exists);
-		}
-	}
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	xmysqlnd_schema* schema = data_object.schema;
+	const st_xmysqlnd_session_on_error_bind on_error{ mysqlx_scheme_on_error, nullptr };
+	schema->exists_in_database(on_error, return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -236,27 +223,30 @@ const enum_hnd_func_status on_drop_db_object_error(
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, createCollection)
 {
-	st_mysqlx_schema* object{nullptr};
+	DBG_ENTER("mysqlx_schema::createCollection");
+
 	zval* object_zv{nullptr};
 	util::string_view collection_name;
-
-	DBG_ENTER("mysqlx_schema::createCollection");
+	const util::string_view Empty_collection_options{ "{}" };
+	util::string_view collection_options(Empty_collection_options);
 	if (FAILURE == util::zend::parse_method_parameters(
-		execute_data, getThis(), "Os",
+		execute_data, getThis(), "Os|s",
 		&object_zv, mysqlx_schema_class_entry,
-		&(collection_name.str), &(collection_name.len)))
+		&(collection_name.str), &(collection_name.len),
+		&collection_options.str, &collection_options.len))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-	RETVAL_FALSE;
-	if (!collection_name.empty() && object->schema) {
-		const struct st_xmysqlnd_schema_on_error_bind on_error = { mysqlx_schema_on_error, nullptr };
 
-		xmysqlnd_collection* const collection = object->schema->create_collection( collection_name, on_error);
+	RETVAL_NULL();
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	if (!collection_name.empty()) {
+		const st_xmysqlnd_schema_on_error_bind on_error{ mysqlx_schema_on_error, nullptr };
+		xmysqlnd_collection* const collection{ data_object.schema->create_collection(collection_name, collection_options, on_error) };
 		DBG_INF_FMT("collection=%p", collection);
 		if (collection) {
-			mysqlx_new_collection(return_value, collection, FALSE);
+			mysqlx_new_collection(return_value, collection);
 			DBG_INF_FMT("type=%d", Z_TYPE_P(return_value));
 			if (Z_TYPE_P(return_value) != IS_OBJECT) {
 				DBG_ERR("Something is wrong");
@@ -267,12 +257,40 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, createCollection)
 	DBG_VOID_RETURN;
 }
 
-MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, dropCollection)
+MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, modifyCollection)
 {
+	DBG_ENTER("mysqlx_schema::modifyCollection");
+
 	zval* object_zv{nullptr};
 	util::string_view collection_name;
+	util::string_view collection_options;
+	if (FAILURE == util::zend::parse_method_parameters(
+		execute_data, getThis(), "Oss",
+		&object_zv, mysqlx_schema_class_entry,
+		&collection_name.str, &collection_name.len,
+		&collection_options.str, &collection_options.len))
+	{
+		DBG_VOID_RETURN;
+	}
 
+	RETVAL_FALSE;
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	if (!collection_name.empty()) {
+		const st_xmysqlnd_schema_on_error_bind on_error{ mysqlx_schema_on_error, nullptr };
+		if (data_object.schema->modify_collection(collection_name, collection_options, on_error)) {
+			RETVAL_TRUE;
+		}
+	}
+	DBG_VOID_RETURN;
+}
+
+MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, dropCollection)
+{
 	DBG_ENTER("mysqlx_schema::dropCollection");
+
+	zval* object_zv{nullptr};
+	util::string_view collection_name;
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data, getThis(), "Os",
 		&object_zv, mysqlx_schema_class_entry,
@@ -284,7 +302,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, dropCollection)
 	auto& data_object = util::fetch_data_object<st_mysqlx_schema>(object_zv);
 
 	try {
-		const st_xmysqlnd_schema_on_error_bind on_error = { on_drop_db_object_error, nullptr };
+		const st_xmysqlnd_schema_on_error_bind on_error{ on_drop_db_object_error, nullptr };
 		RETVAL_BOOL(PASS == data_object.schema->drop_collection( collection_name, on_error));
 	} catch(std::exception& e) {
 		util::log_warning(e.what());
@@ -296,11 +314,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, dropCollection)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getCollection)
 {
-	st_mysqlx_schema* object{nullptr};
+	DBG_ENTER("mysqlx_schema::getCollection");
+
 	zval* object_zv{nullptr};
 	util::string_view collection_name;
-
-	DBG_ENTER("mysqlx_schema::getCollection");
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Os",
 												&object_zv, mysqlx_schema_class_entry,
 												&(collection_name.str), &(collection_name.len)))
@@ -308,12 +325,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getCollection)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-	RETVAL_FALSE;
-	if ( !collection_name.empty() && object->schema) {
-		xmysqlnd_collection* const collection = object->schema->create_collection_object(collection_name.to_nd_cstr());
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	RETVAL_NULL();
+	if ( !collection_name.empty() && data_object.schema) {
+		xmysqlnd_collection* const collection = data_object.schema->create_collection_object(collection_name.to_nd_cstr());
 		if (collection) {
-			mysqlx_new_collection(return_value, collection, FALSE);
+			mysqlx_new_collection(return_value, collection);
 			if (Z_TYPE_P(return_value) != IS_OBJECT) {
 				xmysqlnd_collection_free(collection, nullptr, nullptr);
 			}
@@ -324,11 +341,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getCollection)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getTable)
 {
-	st_mysqlx_schema* object{nullptr};
+	DBG_ENTER("mysqlx_schema::getTable");
+
 	zval* object_zv{nullptr};
 	util::string_view table_name;
-
-	DBG_ENTER("mysqlx_schema::getTable");
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Os",
 												&object_zv, mysqlx_schema_class_entry,
 												&(table_name.str), &(table_name.len)))
@@ -336,11 +352,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getTable)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-	RETVAL_FALSE;
-	if ( !table_name.empty() && object->schema) {
-		xmysqlnd_table* const table = object->schema->create_table_object(table_name.to_nd_cstr());
-		mysqlx_new_table(return_value, table, FALSE /* no clone */);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	RETVAL_NULL();
+	if ( !table_name.empty() && data_object.schema) {
+		xmysqlnd_table* const table = data_object.schema->create_table_object(table_name.to_nd_cstr());
+		mysqlx_new_table(return_value, table);
 		if (Z_TYPE_P(return_value) != IS_OBJECT) {
 			xmysqlnd_table_free(table, nullptr, nullptr);
 		}
@@ -350,22 +366,21 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getTable)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getCollectionAsTable)
 {
-	st_mysqlx_schema* object{nullptr};
-	zval* object_zv{nullptr};
-	MYSQLND_CSTRING collection_name = { nullptr, 0 };
-
 	DBG_ENTER("mysqlx_schema::getCollectionAsTable");
+
+	zval* object_zv{nullptr};
+	util::string_view collection_name;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Os",
 												&object_zv, mysqlx_schema_class_entry,
-												&(collection_name.s), &(collection_name.l)))
+												&(collection_name.str), &(collection_name.len)))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-	RETVAL_FALSE;
-	if (collection_name.s && collection_name.l && object->schema) {
-		xmysqlnd_table* const table = object->schema->create_table_object(collection_name);
-		mysqlx_new_table(return_value, table, FALSE /* no clone */);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	RETVAL_NULL();
+	if (!collection_name.empty() && data_object.schema) {
+		xmysqlnd_table* const table = data_object.schema->create_table_object(collection_name.to_nd_cstr());
+		mysqlx_new_table(return_value, table);
 		if (Z_TYPE_P(return_value) != IS_OBJECT) {
 			xmysqlnd_table_free(table, nullptr, nullptr);
 		}
@@ -391,7 +406,7 @@ mysqlx_on_db_object(void* context, xmysqlnd_schema* const schema, const MYSQLND_
 	if ((object_type.s[0] == 'T') || (object_type.s[0] == 'V')) {
 		xmysqlnd_table* const table = schema->create_table_object(object_name);
 		if (table) {
-			mysqlx_new_table(&zv, table, FALSE);
+			mysqlx_new_table(&zv, table);
 			if (Z_TYPE(zv) == IS_OBJECT) {
 				add_assoc_zval_ex(ctx->list, object_name.s, object_name.l, &zv);
 			} else {
@@ -402,7 +417,7 @@ mysqlx_on_db_object(void* context, xmysqlnd_schema* const schema, const MYSQLND_
 	} else if (object_type.s[0] == 'C') {
 		xmysqlnd_collection* const collection = schema->create_collection_object(object_name);
 		if (collection) {
-			mysqlx_new_collection(&zv, collection, FALSE);
+			mysqlx_new_collection(&zv, collection);
 			if (Z_TYPE(zv) == IS_OBJECT) {
 				add_assoc_zval_ex(ctx->list, object_name.s, object_name.l, &zv);
 			} else {
@@ -442,36 +457,32 @@ mysqlx_get_database_objects(
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getTables)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_schema* object{nullptr};
-
 	DBG_ENTER("mysqlx_schema::getTables");
+
+	zval* object_zv{nullptr};
 	if (util::zend::parse_method_parameters(execute_data, getThis(), "O", &object_zv, mysqlx_schema_class_entry) == FAILURE) {
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-	RETVAL_FALSE;
-
-	mysqlx_get_database_objects(object->schema, drv::db_object_type_filter::table_or_view, return_value);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	RETVAL_NULL();
+	mysqlx_get_database_objects(data_object.schema, drv::db_object_type_filter::table_or_view, return_value);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_schema, getCollections)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_schema* object{nullptr};
-
 	DBG_ENTER("mysqlx_schema::getCollections");
+
+	zval* object_zv{nullptr};
 	if (util::zend::parse_method_parameters(execute_data, getThis(), "O", &object_zv, mysqlx_schema_class_entry) == FAILURE) {
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SCHEMA_FROM_ZVAL(object, object_zv);
-	RETVAL_FALSE;
-
-	mysqlx_get_database_objects(object->schema, drv::db_object_type_filter::collection, return_value);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_schema>(object_zv) };
+	RETVAL_NULL();
+	mysqlx_get_database_objects(data_object.schema, drv::db_object_type_filter::collection, return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -484,6 +495,7 @@ static const zend_function_entry mysqlx_schema_methods[] = {
 	PHP_ME(mysqlx_schema, existsInDatabase, arginfo_mysqlx_schema__exists_in_database, ZEND_ACC_PUBLIC)
 	/************************************** INHERITED END   ****************************************/
 	PHP_ME(mysqlx_schema, createCollection, arginfo_mysqlx_schema__create_collection, ZEND_ACC_PUBLIC)
+	PHP_ME(mysqlx_schema, modifyCollection, arginfo_mysqlx_schema_modify_collection, ZEND_ACC_PUBLIC)
 	PHP_ME(mysqlx_schema, dropCollection, arginfo_mysqlx_schema__drop_collection, ZEND_ACC_PUBLIC)
 	PHP_ME(mysqlx_schema, getCollection, arginfo_mysqlx_schema__get_collection, ZEND_ACC_PUBLIC)
 	PHP_ME(mysqlx_schema, getCollections, arginfo_mysqlx_schema__get_collections, ZEND_ACC_PUBLIC)
@@ -527,20 +539,7 @@ const struct st_mysqlx_property_entry mysqlx_schema_property_entries[] =
 static void
 mysqlx_schema_free_storage(zend_object* object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_schema* inner_obj = static_cast<st_mysqlx_schema*>(mysqlx_object->ptr);
-
-	if (inner_obj) {
-		if (inner_obj->schema) {
-			xmysqlnd_schema_free(inner_obj->schema, nullptr, nullptr);
-			if( inner_obj->schema->get_counter() == 0 ) {
-				mnd_efree(inner_obj->schema);
-			}
-			inner_obj->schema = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_schema>(object);
 }
 
 static zend_object *
@@ -559,21 +558,17 @@ php_mysqlx_schema_object_allocator(zend_class_entry* class_type)
 void
 mysqlx_register_schema_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_schema_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_schema_handlers.free_obj = mysqlx_schema_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Schema", mysqlx_schema_methods);
-		tmp_ce.create_object = php_mysqlx_schema_object_allocator;
-		mysqlx_schema_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_schema_class_entry, 1, mysqlx_database_object_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_schema_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_schema_properties, mysqlx_schema_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_schema_class_entry,
+		"Schema",
+		mysqlx_std_object_handlers,
+		mysqlx_object_schema_handlers,
+		php_mysqlx_schema_object_allocator,
+		mysqlx_schema_free_storage,
+		mysqlx_schema_methods,
+		mysqlx_schema_properties,
+		mysqlx_schema_property_entries,
+		mysqlx_database_object_interface_entry);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_schema_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -590,15 +585,9 @@ mysqlx_new_schema(zval* return_value, xmysqlnd_schema* schema)
 {
 	DBG_ENTER("mysqlx_new_schema");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_schema_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_schema* const object = static_cast<st_mysqlx_schema*>(mysqlx_object->ptr);
-		if (object) {
-			object->schema = schema;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-		}
-	}
+	st_mysqlx_schema& data_object{
+		util::init_object<st_mysqlx_schema>(mysqlx_schema_class_entry, return_value) };
+	data_object.schema = schema;
 
 	DBG_VOID_RETURN;
 }
