@@ -65,16 +65,11 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_sql_statement__get_result, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-#define MYSQLX_FETCH_STATEMENT_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_statement*) mysqlx_object->ptr; \
-	if (!(_to) || (!(_to)->stmt && !(_to)->stmt_execute)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_ERR_FMT("invalid object of class %s on %s::%d", ZSTR_VAL(mysqlx_object->zo.ce->name), __FILE__, __LINE__); \
-		DBG_VOID_RETURN; \
-	} \
-} \
+st_mysqlx_statement::~st_mysqlx_statement()
+{
+	xmysqlnd_stmt_free(stmt, nullptr, nullptr);
+	xmysqlnd_stmt_execute__destroy(stmt_execute);
+}
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement, __construct)
 {
@@ -402,16 +397,17 @@ mysqlx_fetch_data_with_callback(st_mysqlx_statement* object, st_xmysqlnd_exec_wi
 }
 
 void
-mysqlx_sql_statement_bind_one_param(zval * object_zv, const zval * param_zv, zval * return_value)
+mysqlx_sql_statement_bind_one_param(zval* object_zv, const zval* param_zv, zval* return_value)
 {
-	st_mysqlx_statement* object{nullptr};
 	DBG_ENTER("mysqlx_sql_statement_bind_one_param");
-	MYSQLX_FETCH_STATEMENT_FROM_ZVAL(object, object_zv);
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_statement>(object_zv) };
+
 	RETVAL_TRUE;
-	if (TRUE == object->in_execution) {
+	if (TRUE == data_object.in_execution) {
 		php_error_docref(nullptr, E_WARNING, "Statement in execution. Please fetch all data first.");
 		RETVAL_FALSE;
-	} else if (object->stmt_execute && FAIL == xmysqlnd_stmt_execute__bind_one_param_add(object->stmt_execute, param_zv)) {
+	} else if (data_object.stmt_execute && FAIL == xmysqlnd_stmt_execute__bind_one_param_add(data_object.stmt_execute, param_zv)) {
 		RETVAL_FALSE;
 	}
 	DBG_VOID_RETURN;
@@ -467,7 +463,7 @@ mysqlx_sql_stmt_on_error(
 void
 mysqlx_sql_statement_get_results(
 		st_mysqlx_statement* object,
-		zval * return_value
+		zval* return_value
 )
 {
 	DBG_ENTER("mysqlx_sql_statement_get_results");
@@ -506,7 +502,7 @@ void
 mysqlx_sql_statement_execute(
 		const st_mysqlx_object* const mysqlx_object,
 		const zend_long flags,
-		zval * return_value
+		zval* return_value
 )
 {
 	st_mysqlx_statement* object = (st_mysqlx_statement*) mysqlx_object->ptr;
@@ -566,19 +562,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement, execute)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement, hasMoreResults)
 {
-	st_mysqlx_statement* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement::hasMoreResults");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_STATEMENT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_statement>(object_zv) };
 
-	RETVAL_BOOL(object->stmt->has_more_results(object->stmt));
+	RETVAL_BOOL(data_object.stmt->has_more_results(data_object.stmt));
 	DBG_INF_FMT("%s", Z_TYPE_P(return_value) == IS_TRUE? "YES":"NO");
 
 	DBG_VOID_RETURN;
@@ -586,12 +581,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement, hasMoreResults)
 
 static void mysqlx_sql_statement_read_result(INTERNAL_FUNCTION_PARAMETERS, zend_class_entry * const class_entry)
 {
-	st_mysqlx_statement* object{nullptr};
+	DBG_ENTER("mysqlx_sql_statement_read_result");
+
 	zval* object_zv{nullptr};
 	zend_bool use_callbacks{FALSE};
 	st_xmysqlnd_exec_with_cb_ctx xmysqlnd_exec_with_cb_ctx{};
-
-	DBG_ENTER("mysqlx_sql_statement_read_result");
 	if (ZEND_NUM_ARGS() == 0) {
 		if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 													&object_zv, class_entry))
@@ -613,32 +607,32 @@ static void mysqlx_sql_statement_read_result(INTERNAL_FUNCTION_PARAMETERS, zend_
 		use_callbacks = TRUE;
 	}
 
-	MYSQLX_FETCH_STATEMENT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_statement>(object_zv) };
 
 	RETVAL_FALSE;
-	if (PASS == object->send_query_status) {
-		xmysqlnd_stmt * stmt = object->stmt;
+	if (PASS == data_object.send_query_status) {
+		xmysqlnd_stmt * stmt = data_object.stmt;
 
 		if (use_callbacks) {
-			RETVAL_BOOL(PASS == mysqlx_fetch_data_with_callback(object, &xmysqlnd_exec_with_cb_ctx));
+			RETVAL_BOOL(PASS == mysqlx_fetch_data_with_callback(&data_object, &xmysqlnd_exec_with_cb_ctx));
 		} else {
 			const struct st_xmysqlnd_stmt_on_warning_bind on_warning = { mysqlx_sql_stmt_on_warning, nullptr };
 			const struct st_xmysqlnd_stmt_on_error_bind on_error = { mysqlx_sql_stmt_on_error, nullptr };
 			XMYSQLND_STMT_RESULT * result;
 
-			if (object->execute_flags & MYSQLX_EXECUTE_FLAG_BUFFERED) {
-				result = object->stmt->get_buffered_result(stmt, &object->has_more_results, on_warning, on_error, nullptr, nullptr);
+			if (data_object.execute_flags & MYSQLX_EXECUTE_FLAG_BUFFERED) {
+				result = data_object.stmt->get_buffered_result(stmt, &data_object.has_more_results, on_warning, on_error, nullptr, nullptr);
 			} else {
-				result = object->stmt->get_fwd_result(stmt, MYSQLX_EXECUTE_FWD_PREFETCH_COUNT, &object->has_more_rows_in_set, &object->has_more_results, on_warning, on_error, nullptr, nullptr);
+				result = data_object.stmt->get_fwd_result(stmt, MYSQLX_EXECUTE_FWD_PREFETCH_COUNT, &data_object.has_more_rows_in_set, &data_object.has_more_results, on_warning, on_error, nullptr, nullptr);
 			}
 
-			DBG_INF_FMT("result=%p  has_more_results=%s", result, object->has_more_results? "TRUE":"FALSE");
+			DBG_INF_FMT("result=%p  has_more_results=%s", result, data_object.has_more_results? "TRUE":"FALSE");
 			if (result) {
-				mysqlx_new_sql_stmt_result(return_value, result, object);
+				mysqlx_new_sql_stmt_result(return_value, result, &data_object);
 			} else {
 				RAISE_EXCEPTION_FETCH_FAIL();
 				/* Or we should close the connection, rendering it unusable at this point ?*/
-				object->send_query_status = FAIL;
+				data_object.send_query_status = FAIL;
 			}
 		}
 	}
@@ -679,25 +673,11 @@ const struct st_mysqlx_property_entry mysqlx_sql_statement_property_entries[] =
 static void
 mysqlx_sql_statement_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_statement* inner_obj = (st_mysqlx_statement*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->stmt) {
-			xmysqlnd_stmt_free(inner_obj->stmt, nullptr, nullptr);
-			inner_obj->stmt = nullptr;
-		}
-		if (inner_obj->stmt_execute) {
-			xmysqlnd_stmt_execute__destroy(inner_obj->stmt_execute);
-			inner_obj->stmt_execute = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_statement>(object);
 }
 
 static zend_object *
-php_mysqlx_sql_statement_object_allocator(zend_class_entry * class_type)
+php_mysqlx_sql_statement_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_sql_statement_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_statement>(
@@ -708,22 +688,18 @@ php_mysqlx_sql_statement_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_sql_statement_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_sql_statement_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_sql_statement_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_sql_statement_handlers.free_obj = mysqlx_sql_statement_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "SqlStatement", mysqlx_sql_statement_methods);
-		tmp_ce.create_object = php_mysqlx_sql_statement_object_allocator;
-		mysqlx_sql_statement_class_entry = zend_register_internal_class(&tmp_ce);
-	}
-
-	zend_hash_init(&mysqlx_sql_statement_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_sql_statement_properties, mysqlx_sql_statement_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_sql_statement_class_entry,
+		"SqlStatement",
+		mysqlx_std_object_handlers,
+		mysqlx_object_sql_statement_handlers,
+		php_mysqlx_sql_statement_object_allocator,
+		mysqlx_sql_statement_free_storage,
+		mysqlx_sql_statement_methods,
+		mysqlx_sql_statement_properties,
+		mysqlx_sql_statement_property_entries);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_sql_statement_class_entry, "statement",	sizeof("statement") - 1, ZEND_ACC_PUBLIC);
@@ -739,32 +715,20 @@ mysqlx_unregister_sql_statement_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_sql_stmt(zval * return_value, xmysqlnd_stmt * stmt, const MYSQLND_CSTRING namespace_, const MYSQLND_CSTRING query)
+mysqlx_new_sql_stmt(zval* return_value, xmysqlnd_stmt* stmt, const MYSQLND_CSTRING namespace_, const MYSQLND_CSTRING query)
 {
 	DBG_ENTER("mysqlx_new_sql_stmt");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_sql_statement_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_statement* object = (st_mysqlx_statement*) mysqlx_object->ptr;
-		XMYSQLND_STMT_OP__EXECUTE * stmt_execute = xmysqlnd_stmt_execute__create(namespace_, query);
+	st_mysqlx_statement& data_object{
+		util::init_object<st_mysqlx_statement>(mysqlx_sql_statement_class_entry, return_value) };
+	data_object.stmt = stmt;
+	data_object.stmt_execute = xmysqlnd_stmt_execute__create(namespace_, query);
+	data_object.execute_flags = 0;
+	data_object.send_query_status = FAIL;
+	data_object.in_execution = FALSE;
+	data_object.has_more_results = FALSE;
+	data_object.has_more_rows_in_set = FALSE;
 
-		if (object && stmt && stmt_execute) {
-			object->stmt = stmt;
-			object->stmt_execute = stmt_execute;
-			object->execute_flags = 0;
-			object->send_query_status = FAIL;
-			object->in_execution = FALSE;
-			object->has_more_results = FALSE;
-			object->has_more_rows_in_set = FALSE;
-		} else {
-			if (stmt_execute) {
-				xmysqlnd_stmt_execute__destroy(stmt_execute);
-			}
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
 	DBG_VOID_RETURN;
 }
 
@@ -773,7 +737,7 @@ void
 mysqlx_statement_execute_read_response(const st_mysqlx_object* const mysqlx_object,
 											const zend_long flags,
 											const enum mysqlx_result_type result_type,
-											zval * return_value)
+											zval* return_value)
 {
 	st_mysqlx_statement* object = (st_mysqlx_statement*) mysqlx_object->ptr;
 	DBG_ENTER("mysqlx_statement_execute");
@@ -896,19 +860,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_statement, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_statement, hasMoreResults)
 {
-	st_mysqlx_statement* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_statement::hasMoreResults");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_statement_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_STATEMENT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_statement>(object_zv) };
 
-	RETVAL_BOOL(object->stmt->has_more_results(object->stmt));
+	RETVAL_BOOL(data_object.stmt->has_more_results(data_object.stmt));
 	DBG_INF_FMT("%s", Z_TYPE_P(return_value) == IS_TRUE? "YES":"NO");
 
 	DBG_VOID_RETURN;
@@ -950,29 +913,25 @@ mysqlx_statement_free_storage(zend_object * object)
 }
 
 static zend_object *
-php_mysqlx_statement_object_allocator(zend_class_entry * class_type)
+php_mysqlx_statement_object_allocator(zend_class_entry* class_type)
 {
 	/* shows the intention that we do reuse code */
 	return php_mysqlx_sql_statement_object_allocator(class_type);
 }
 
 void
-mysqlx_register_statement_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_statement_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_statement_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_statement_handlers.free_obj = mysqlx_statement_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Statement", mysqlx_statement_methods);
-		tmp_ce.create_object = php_mysqlx_statement_object_allocator;
-		mysqlx_statement_class_entry = zend_register_internal_class(&tmp_ce);
-	}
-
-	zend_hash_init(&mysqlx_statement_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_statement_properties, mysqlx_statement_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_statement_class_entry,
+		"Statement",
+		mysqlx_std_object_handlers,
+		mysqlx_object_statement_handlers,
+		php_mysqlx_statement_object_allocator,
+		mysqlx_statement_free_storage,
+		mysqlx_statement_methods,
+		mysqlx_statement_properties,
+		mysqlx_statement_property_entries);
 
 	zend_declare_class_constant_long(mysqlx_statement_class_entry, "EXECUTE_ASYNC", sizeof("EXECUTE_ASYNC") - 1, MYSQLX_EXECUTE_FLAG_ASYNC);
 	zend_declare_class_constant_long(mysqlx_statement_class_entry, "BUFFERED", sizeof("BUFFERED") - 1, MYSQLX_EXECUTE_FLAG_BUFFERED);
@@ -985,27 +944,20 @@ mysqlx_unregister_statement_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_stmt(zval * return_value, xmysqlnd_stmt * stmt)
+mysqlx_new_stmt(zval* return_value, xmysqlnd_stmt * stmt)
 {
 	DBG_ENTER("mysqlx_new_stmt");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_statement_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_statement* object = (st_mysqlx_statement*) mysqlx_object->ptr;
-		if (object) {
-			object->stmt = stmt;
-			object->stmt_execute = nullptr;
-			object->execute_flags = 0;
-			object->send_query_status = FAIL;
-			object->in_execution = FALSE;
-			object->has_more_results = FALSE;
-			object->has_more_rows_in_set = FALSE;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_statement& data_object{
+		util::init_object<st_mysqlx_statement>(mysqlx_statement_class_entry, return_value) };
+	data_object.stmt = stmt;
+	data_object.stmt_execute = nullptr;
+	data_object.execute_flags = 0;
+	data_object.send_query_status = FAIL;
+	data_object.in_execution = FALSE;
+	data_object.has_more_results = FALSE;
+	data_object.has_more_rows_in_set = FALSE;
+
 	DBG_VOID_RETURN;
 }
 
