@@ -84,17 +84,11 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlx_sql_statement_result__next_result, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-#define MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_sql_statement_result*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
-} \
-
+st_mysqlx_sql_statement_result::~st_mysqlx_sql_statement_result()
+{
+	xmysqlnd_stmt_free(stmt, nullptr, nullptr);
+	xmysqlnd_stmt_result_free(result, nullptr, nullptr);
+}
 
 static const enum_hnd_func_status
 mysqlx_sql_stmt_result_on_warning(
@@ -163,44 +157,37 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, hasData)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::hasData");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
-	RETVAL_BOOL(object && object->result && FALSE == object->result->m.eof(object->result));
+	RETVAL_BOOL(FALSE == data_object.result->m.eof(data_object.result));
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, fetchOne)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
-	RETVAL_NULL();
-
 	DBG_ENTER("mysqlx_sql_statement_result::fetchOne");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	RETVAL_NULL();
 
-	if (object && object->result && FALSE == object->result->m.eof(object->result)) {
-		zval row;
-		ZVAL_UNDEF(&row);
-		if (PASS == object->result->m.fetch_current(object->result, &row, nullptr, nullptr)) {
-			ZVAL_COPY_VALUE(return_value, &row);
-			object->result->m.next(object->result, nullptr, nullptr);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
+	auto& obj_result{ data_object.result };
+	if (FALSE == obj_result->m.eof(obj_result)) {
+		if (PASS == obj_result->m.fetch_current(obj_result, return_value, nullptr, nullptr)) {
+			obj_result->m.next(obj_result, nullptr, nullptr);
 		}
 	}
 	DBG_VOID_RETURN;
@@ -219,11 +206,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, fetchAll)
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 	if (data_object.result) {
-		zval set;
-		ZVAL_UNDEF(&set);
-		if (PASS == data_object.result->m.fetch_all(data_object.result, &set, nullptr, nullptr)) {
-			ZVAL_COPY_VALUE(return_value, &set);
-		}
+		data_object.result->m.fetch_all(data_object.result, return_value, nullptr, nullptr);
 	}
 	util::zend::ensure_is_array(return_value);
 
@@ -232,30 +215,26 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, fetchAll)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getAffectedItemsCount)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::getAffectedItemsCount");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
 	RETVAL_LONG(0);
-	if (object->result) {
-		const XMYSQLND_STMT_EXECUTION_STATE * exec_state = object->result->exec_state;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (exec_state) {
-			const size_t value = exec_state->m->get_affected_items_count(exec_state);
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				RETVAL_LONG(value);
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+	const XMYSQLND_STMT_EXECUTION_STATE* exec_state = data_object.result->exec_state;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (exec_state) {
+		const size_t value = exec_state->m->get_affected_items_count(exec_state);
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			RETVAL_LONG(value);
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -263,20 +242,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getAffectedItemsCount)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getLastInsertId)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::getLastInsertId");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
 	RETVAL_FALSE;
-	if (object->result && object->result->exec_state) {
-		const XMYSQLND_STMT_EXECUTION_STATE * const exec_state = object->result->exec_state;
+	const XMYSQLND_STMT_EXECUTION_STATE* const exec_state = data_object.result->exec_state;
+	if (exec_state) {
 		/* Maybe check here if there was an error and throw an Exception or return a warning */
 		if (exec_state) {
 			const uint64_t value = exec_state->m->get_last_insert_id(exec_state);
@@ -294,64 +271,52 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getLastInsertId)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getGeneratedIds)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::getGeneratedIds");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
-	if (object->result && object->result->exec_state) {
-		const XMYSQLND_STMT_EXECUTION_STATE * const exec_state = object->result->exec_state;
-		if ( exec_state == nullptr ) {
-			php_error_docref(nullptr, E_WARNING, "Unable to get the correct exec_state");
-			DBG_VOID_RETURN;
-		}
-		auto& ids = exec_state->generated_doc_ids;
-		const size_t num_of_docs = ids.size();
-		array_init_size(return_value, static_cast<uint32_t>(num_of_docs));
-		for( auto& elem : ids ) {
-			zval id;
-			ZVAL_STRINGL(&id,elem.c_str(),elem.size());
-			zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &id);
-		}
+	const XMYSQLND_STMT_EXECUTION_STATE* const exec_state = data_object.result->exec_state;
+	if ( exec_state == nullptr ) {
+		php_error_docref(nullptr, E_WARNING, "Unable to get the correct exec_state");
+		DBG_VOID_RETURN;
 	}
-
-	util::zend::ensure_is_array(return_value);
+	auto& ids = exec_state->generated_doc_ids;
+	util::zvalue docs(util::zvalue::create_array(ids.size()));
+	for( auto& elem : ids ) {
+		docs.push_back(elem);
+	}
+	docs.move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getWarningsCount)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::getWarningsCount");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
 	RETVAL_LONG(0);
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const size_t value = warnings->m->count(warnings);
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				RETVAL_LONG(value);
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+	const XMYSQLND_WARNING_LIST* const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const size_t value = warnings->m->count(warnings);
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			RETVAL_LONG(value);
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -359,33 +324,28 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getWarningsCount)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getWarnings)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::getWarnings");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
-	if (object->result) {
-		const XMYSQLND_WARNING_LIST * const warnings = object->result->warnings;
-		/* Maybe check here if there was an error and throw an Exception or return a warning */
-		if (warnings) {
-			const unsigned int count{warnings->m->count(warnings)};
-			array_init_size(return_value, count);
-			for (unsigned int i{0}; i < count; ++i) {
-				const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
-				zval warning_zv;
+	const XMYSQLND_WARNING_LIST * const warnings = data_object.result->warnings;
+	/* Maybe check here if there was an error and throw an Exception or return a warning */
+	if (warnings) {
+		const unsigned int count{warnings->m->count(warnings)};
+		array_init_size(return_value, count);
+		for (unsigned int i{0}; i < count; ++i) {
+			const XMYSQLND_WARNING warning = warnings->m->get_warning(warnings, i);
+			util::zvalue warning_zv;
+			mysqlx_new_warning(warning_zv.ptr(), warning.message, warning.level, warning.code);
 
-				ZVAL_UNDEF(&warning_zv);
-				mysqlx_new_warning(&warning_zv, warning.message, warning.level, warning.code);
-
-				if (Z_TYPE(warning_zv) != IS_UNDEF) {
-					zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &warning_zv);
-				}
+			if (!warning_zv.is_undef()) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), warning_zv.ptr());
+				warning_zv.invalidate();
 			}
 		}
 	}
@@ -431,18 +391,15 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getColumnsCount)
 	RETVAL_LONG(0);
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
-	if (data_object.result) {
-		st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
-		if (meta)
-		{
-			const size_t value{ meta->m->get_field_count(meta) };
-			if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
-				ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
-				DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
-			} else {
-				ZVAL_LONG(return_value, value);
-				DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
-			}
+	st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
+	if (meta) {
+		const size_t value{ meta->m->get_field_count(meta) };
+		if (UNEXPECTED(value >= ZEND_LONG_MAX)) {
+			ZVAL_NEW_STR(return_value, strpprintf(0, "%s", util::to_string(value).c_str()));
+			DBG_INF_FMT("value(S)=%s", Z_STRVAL_P(return_value));
+		} else {
+			ZVAL_LONG(return_value, value);
+			DBG_INF_FMT("value(L)=%lu", Z_LVAL_P(return_value));
 		}
 	}
 	DBG_VOID_RETURN;
@@ -460,22 +417,20 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getColumns)
 	}
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
-	if (data_object.result) {
-		st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
-		/* Maybe check here if there was an error and throw an Exception or return a column */
-		if (meta) {
-			const unsigned int count{meta->m->get_field_count(meta)};
-			array_init_size(return_value, count);
-			for (unsigned int i{0}; i < count; ++i) {
-				const XMYSQLND_RESULT_FIELD_META* column = meta->m->get_field(meta, i);
-				zval column_zv;
+	st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
+	/* Maybe check here if there was an error and throw an Exception or return a column */
+	if (meta) {
+		const unsigned int count{meta->m->get_field_count(meta)};
+		array_init_size(return_value, count);
+		for (unsigned int i{0}; i < count; ++i) {
+			const XMYSQLND_RESULT_FIELD_META* column = meta->m->get_field(meta, i);
+			zval column_zv;
 
-				ZVAL_UNDEF(&column_zv);
-				mysqlx_new_column_result(&column_zv, column);
+			ZVAL_UNDEF(&column_zv);
+			mysqlx_new_column_result(&column_zv, column);
 
-				if (Z_TYPE(column_zv) != IS_UNDEF) {
-					zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &column_zv);
-				}
+			if (Z_TYPE(column_zv) != IS_UNDEF) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &column_zv);
 			}
 		}
 	}
@@ -497,22 +452,20 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getColumnNames)
 	}
 
 	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
-	if (data_object.result) {
-		st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
-		/* Maybe check here if there was an error and throw an Exception or return a column */
-		if (meta) {
-			const unsigned int count{meta->m->get_field_count(meta)};
-			array_init_size(return_value, count);
-			for (unsigned int i{0}; i < count; ++i) {
-				const XMYSQLND_RESULT_FIELD_META* column{ meta->m->get_field(meta, i) };
-				zval column_name;
+	st_xmysqlnd_stmt_result_meta* meta{ get_stmt_result_meta(data_object.result) };
+	/* Maybe check here if there was an error and throw an Exception or return a column */
+	if (meta) {
+		const unsigned int count{meta->m->get_field_count(meta)};
+		array_init_size(return_value, count);
+		for (unsigned int i{0}; i < count; ++i) {
+			const XMYSQLND_RESULT_FIELD_META* column{ meta->m->get_field(meta, i) };
+			zval column_name;
 
-				ZVAL_UNDEF(&column_name);
-				ZVAL_STRINGL(&column_name, column->name.s, column->name.l);
+			ZVAL_UNDEF(&column_name);
+			ZVAL_STRINGL(&column_name, column->name.s, column->name.l);
 
-				if (Z_TYPE(column_name) != IS_UNDEF) {
-					zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &column_name);
-				}
+			if (Z_TYPE(column_name) != IS_UNDEF) {
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &column_name);
 			}
 		}
 	}
@@ -524,20 +477,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, getColumnNames)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_sql_statement_result, nextResult)
 {
-	zval* object_zv{nullptr};
-	st_mysqlx_sql_statement_result* object{nullptr};
-
 	DBG_ENTER("mysqlx_sql_statement_result::nextResult");
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_sql_statement_result_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
-	MYSQLX_FETCH_SQL_STATEMENT_RESULT_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_sql_statement_result>(object_zv) };
 
 	RETVAL_FALSE;
-	if (object->result && object->has_more_results) {
-		if (mysqlx_sql_statement_read_next_result(object)) {
+	if (data_object.has_more_results) {
+		if (mysqlx_sql_statement_read_next_result(&data_object)) {
 			RETVAL_TRUE;
 		}
 	}
@@ -574,23 +525,11 @@ const struct st_mysqlx_property_entry mysqlx_sql_statement_result_property_entri
 static void
 mysqlx_sql_statement_result_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_sql_statement_result* inner_obj = (st_mysqlx_sql_statement_result*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->stmt) {
-			xmysqlnd_stmt_free(inner_obj->stmt, nullptr, nullptr);
-		}
-		if (inner_obj->result) {
-			xmysqlnd_stmt_result_free(inner_obj->result, nullptr, nullptr);
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_sql_statement_result>(object);
 }
 
 static zend_object *
-php_mysqlx_sql_statement_result_object_allocator(zend_class_entry * class_type)
+php_mysqlx_sql_statement_result_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_sql_statement_result_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_sql_statement_result>(
@@ -601,25 +540,21 @@ php_mysqlx_sql_statement_result_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_sql_statement_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_sql_statement_result_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_sql_statement_result_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_sql_statement_result_handlers.free_obj = mysqlx_sql_statement_result_free_storage;
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "SqlStatementResult", mysqlx_sql_statement_result_methods);
-		tmp_ce.create_object = php_mysqlx_sql_statement_result_object_allocator;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_sql_statement_result_class_entry,
+		"SqlStatementResult",
+		mysqlx_std_object_handlers,
+		mysqlx_object_sql_statement_result_handlers,
+		php_mysqlx_sql_statement_result_object_allocator,
+		mysqlx_sql_statement_result_free_storage,
+		mysqlx_sql_statement_result_methods,
+		mysqlx_sql_statement_result_properties,
+		mysqlx_sql_statement_result_property_entries,
+		mysqlx_base_result_interface_entry);
 
-		mysqlx_sql_statement_result_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_sql_statement_result_class_entry, 1, mysqlx_base_result_interface_entry);
-
-		mysqlx_register_sql_statement_result_iterator(mysqlx_sql_statement_result_class_entry);
-	}
-
-	zend_hash_init(&mysqlx_sql_statement_result_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_sql_statement_result_properties, mysqlx_sql_statement_result_property_entries);
+	mysqlx_register_sql_statement_result_iterator(mysqlx_sql_statement_result_class_entry);
 }
 
 void
@@ -629,27 +564,19 @@ mysqlx_unregister_sql_statement_result_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_sql_stmt_result(zval * return_value, XMYSQLND_STMT_RESULT * result, st_mysqlx_statement* stmt)
+mysqlx_new_sql_stmt_result(zval* return_value, XMYSQLND_STMT_RESULT* result, st_mysqlx_statement* stmt)
 {
 	DBG_ENTER("mysqlx_new_sql_stmt_result");
 
-	if (SUCCESS == object_init_ex(return_value, mysqlx_sql_statement_result_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_sql_statement_result* const object = (st_mysqlx_sql_statement_result*) mysqlx_object->ptr;
-		if (object) {
-			object->result = result;
-			stmt->stmt->get_reference(stmt->stmt);
-			object->stmt = stmt->stmt;
-			object->execute_flags = stmt->execute_flags;
-			object->send_query_status = stmt->send_query_status;
-			object->has_more_results = stmt->has_more_results;
-			object->has_more_rows_in_set = stmt->has_more_rows_in_set;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_sql_statement_result& data_object{
+		util::init_object<st_mysqlx_sql_statement_result>(mysqlx_sql_statement_result_class_entry, return_value) };
+	data_object.result = result;
+	stmt->stmt->get_reference(stmt->stmt);
+	data_object.stmt = stmt->stmt;
+	data_object.execute_flags = stmt->execute_flags;
+	data_object.send_query_status = stmt->send_query_status;
+	data_object.has_more_results = stmt->has_more_results;
+	data_object.has_more_rows_in_set = stmt->has_more_rows_in_set;
 
 	DBG_VOID_RETURN;
 }
