@@ -89,19 +89,15 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_table : public util::custom_allocable
 {
-	xmysqlnd_table * table;
+	~st_mysqlx_table();
+	xmysqlnd_table* table;
 };
 
+st_mysqlx_table::~st_mysqlx_table()
+{
+	xmysqlnd_table_free(table, nullptr, nullptr);
+}
 
-#define MYSQLX_FETCH_TABLE_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_table*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->table) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, __construct)
 {
@@ -136,23 +132,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSession)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getName)
 {
-	st_mysqlx_table* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table::getName");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 
-	if (object->table) {
-		RETVAL_STRINGL(object->table->get_name().s, object->table->get_name().l);
-	} else {
-		RETVAL_FALSE;
-	}
+	RETVAL_STRINGL(data_object.table->get_name().s, data_object.table->get_name().l);
 
 	DBG_VOID_RETURN;
 }
@@ -178,28 +169,25 @@ mysqlx_table_on_error(
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, existsInDatabase)
 {
-	st_mysqlx_table* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table::existsInDatabase");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 
 	RETVAL_FALSE;
 
-	xmysqlnd_table * table = object->table;
-	if (table) {
-		const struct st_xmysqlnd_session_on_error_bind on_error = { mysqlx_table_on_error, nullptr };
-		zval exists;
-		ZVAL_UNDEF(&exists);
-		if (PASS == table->exists_in_database(on_error, &exists)) {
-			ZVAL_COPY_VALUE(return_value, &exists);
-		}
+	xmysqlnd_table* table = data_object.table;
+	const st_xmysqlnd_session_on_error_bind on_error = { mysqlx_table_on_error, nullptr };
+	zval exists;
+	ZVAL_UNDEF(&exists);
+	if (PASS == table->exists_in_database(on_error, &exists)) {
+		ZVAL_COPY_VALUE(return_value, &exists);
 	}
 
 	DBG_VOID_RETURN;
@@ -207,11 +195,9 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, existsInDatabase)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, isView)
 {
-	zval* object_zv{nullptr};
-
-	RETVAL_FALSE;
-
 	DBG_ENTER("mysqlx_table::isView");
+
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data,
 		getThis(),
@@ -224,14 +210,14 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, isView)
 
 	auto& data_object = util::fetch_data_object<st_mysqlx_table>(object_zv);
 
+	RETVAL_FALSE;
+
 	xmysqlnd_table * table = data_object.table;
-	if (table) {
-		const st_xmysqlnd_session_on_error_bind on_error = { mysqlx_table_on_error, nullptr };
-		zval exists;
-		ZVAL_UNDEF(&exists);
-		if (PASS == table->is_view(on_error, &exists)) {
-			ZVAL_COPY_VALUE(return_value, &exists);
-		}
+	const st_xmysqlnd_session_on_error_bind on_error = { mysqlx_table_on_error, nullptr };
+	zval exists;
+	ZVAL_UNDEF(&exists);
+	if (PASS == table->is_view(on_error, &exists)) {
+		ZVAL_COPY_VALUE(return_value, &exists);
 	}
 
 	DBG_VOID_RETURN;
@@ -266,12 +252,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, count)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSchema)
 {
-	st_mysqlx_table* object{nullptr};
-	XMYSQLND_SESSION session;
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table::getSchema");
 
+	XMYSQLND_SESSION session;
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(
 				execute_data,
 				getThis(), "O",
@@ -280,17 +264,16 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSchema)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 	RETVAL_FALSE;
 
-	if( object->table &&
-		object->table->get_schema() ) {
-		session = object->table->get_schema()->get_session();
+	if( data_object.table->get_schema() ) {
+		session = data_object.table->get_schema()->get_session();
 	}
 
 	if(session != nullptr) {
-		MYSQLND_STRING schema_name{ object->table->get_schema()->get_name() };
-		xmysqlnd_schema * schema = session->create_schema_object(
+		MYSQLND_STRING schema_name{ data_object.table->get_schema()->get_name() };
+		xmysqlnd_schema* schema = session->create_schema_object(
 					mnd_str2c(schema_name));
 		if (schema) {
 			mysqlx_new_schema(return_value, schema);
@@ -308,13 +291,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, getSchema)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, insert)
 {
-	st_mysqlx_table* object{nullptr};
+	DBG_ENTER("mysqlx_table::insert");
+
 	zval* object_zv{nullptr};
 	zval* columns{nullptr};
 	int num_of_columns{0};
-
-	DBG_ENTER("mysqlx_table::insert");
-
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data, getThis(), "O+",
 		&object_zv, mysqlx_table_class_entry,
@@ -338,14 +319,13 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, insert)
 		}
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->table && num_of_columns > 0) {
+	if (num_of_columns > 0) {
 		mysqlx_new_table__insert(return_value,
-									  object->table,
-									  TRUE /* clone */,
+									  data_object.table,
 									  columns,
 									  num_of_columns);
 	}
@@ -355,13 +335,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, insert)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, select)
 {
-	st_mysqlx_table* object{nullptr};
+	DBG_ENTER("mysqlx_table::select");
+
 	zval* object_zv{nullptr};
 	zval* columns{nullptr};
 	int num_of_columns{0};
-
-	DBG_ENTER("mysqlx_table::select");
-
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data, getThis(), "O+",
 		&object_zv, mysqlx_table_class_entry,
@@ -371,16 +349,15 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, select)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->table && columns) {
+	if (columns) {
 		DBG_INF_FMT("Num of columns: %d",
 					num_of_columns);
 		mysqlx_new_table__select(return_value,
-						object->table,
-						TRUE /* clone */,
+						data_object.table,
 						columns,
 						num_of_columns);
 	}
@@ -390,48 +367,40 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, select)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, update)
 {
-	st_mysqlx_table* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table::update");
 
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->table) {
-		mysqlx_new_table__update(return_value, object->table, TRUE /* clone */);
-	}
+	mysqlx_new_table__update(return_value, data_object.table);
 
 	DBG_VOID_RETURN;
 }
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table, delete)
 {
-	st_mysqlx_table* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table::delete");
 
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->table) {
-		mysqlx_new_table__delete(return_value, object->table, TRUE /* clone */);
-	}
+	mysqlx_new_table__delete(return_value, data_object.table);
 
 	DBG_VOID_RETURN;
 }
@@ -457,7 +426,7 @@ static const zend_function_entry mysqlx_table_methods[] = {
 };
 
 static zval *
-mysqlx_table_property__name(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_table_property__name(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_table* object = (const st_mysqlx_table* ) (obj->ptr);
 	DBG_ENTER("mysqlx_table_property__name");
@@ -479,7 +448,7 @@ mysqlx_table_property__name(const st_mysqlx_object* obj, zval * return_value)
 static zend_object_handlers mysqlx_object_table_handlers;
 static HashTable mysqlx_table_properties;
 
-const struct st_mysqlx_property_entry mysqlx_table_property_entries[] =
+const st_mysqlx_property_entry mysqlx_table_property_entries[] =
 {
 	{{"name",	sizeof("name") - 1}, mysqlx_table_property__name,	nullptr},
 	{{nullptr,	0}, nullptr, nullptr}
@@ -488,21 +457,11 @@ const struct st_mysqlx_property_entry mysqlx_table_property_entries[] =
 static void
 mysqlx_table_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_table* inner_obj = (st_mysqlx_table*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->table) {
-			xmysqlnd_table_free(inner_obj->table, nullptr, nullptr);
-			inner_obj->table = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_table>(object);
 }
 
 static zend_object *
-php_mysqlx_table_object_allocator(zend_class_entry * class_type)
+php_mysqlx_table_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_table_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_table>(
@@ -513,23 +472,19 @@ php_mysqlx_table_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_table_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_table_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_table_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_table_handlers.free_obj = mysqlx_table_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Table", mysqlx_table_methods);
-		tmp_ce.create_object = php_mysqlx_table_object_allocator;
-		mysqlx_table_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_table_class_entry, 1, mysqlx_schema_object_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_table_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_table_properties, mysqlx_table_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_table_class_entry,
+		"Table",
+		mysqlx_std_object_handlers,
+		mysqlx_object_table_handlers,
+		php_mysqlx_table_object_allocator,
+		mysqlx_table_free_storage,
+		mysqlx_table_methods,
+		mysqlx_table_properties,
+		mysqlx_table_property_entries,
+		mysqlx_schema_object_interface_entry);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_table_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -542,22 +497,12 @@ mysqlx_unregister_table_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_table(zval * return_value, xmysqlnd_table * table, const zend_bool clone)
+mysqlx_new_table(zval* return_value, xmysqlnd_table* table)
 {
 	DBG_ENTER("mysqlx_new_table");
-
-	if (SUCCESS == object_init_ex(return_value, mysqlx_table_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_table* const object = (st_mysqlx_table*) mysqlx_object->ptr;
-		if (object) {
-			object->table = clone? table->get_reference() : table;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
-
+	st_mysqlx_table& data_object{
+		util::init_object<st_mysqlx_table>(mysqlx_table_class_entry, return_value) };
+	data_object.table = table;
 	DBG_VOID_RETURN;
 }
 
