@@ -66,20 +66,16 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_table__delete : public util::custom_allocable
 {
-	XMYSQLND_CRUD_TABLE_OP__DELETE * crud_op;
-	xmysqlnd_table * table;
+	~st_mysqlx_table__delete();
+	XMYSQLND_CRUD_TABLE_OP__DELETE* crud_op;
+	xmysqlnd_table* table;
 };
 
-
-#define MYSQLX_FETCH_TABLE_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_table__delete*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->table) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
+st_mysqlx_table__delete::~st_mysqlx_table__delete()
+{
+	xmysqlnd_table_free(table, nullptr, nullptr);
+	xmysqlnd_crud_table_delete__destroy(crud_op);
+}
 
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, __construct)
@@ -89,12 +85,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, where)
 {
-	st_mysqlx_table__delete* object{nullptr};
-	zval* object_zv{nullptr};
-	zval* where_expr{nullptr};
-
 	DBG_ENTER("mysqlx_table__delete::where");
 
+	zval* object_zv{nullptr};
+	zval* where_expr{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Oz",
 		&object_zv, mysqlx_table__delete_class_entry,
 		&where_expr))
@@ -102,20 +96,15 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, where)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->crud_op && where_expr)
-	{
-
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__delete>(object_zv) };
+	if (data_object.crud_op && where_expr) {
 		if(Z_TYPE_P(where_expr) != IS_STRING) {
 			RAISE_EXCEPTION(err_msg_wrong_param_4);
-		}
-		else
-		{
+		} else {
 			const MYSQLND_CSTRING where_expr_str = {Z_STRVAL_P(where_expr), Z_STRLEN_P(where_expr)};
-			if (PASS == xmysqlnd_crud_table_delete__set_criteria(object->crud_op, where_expr_str)) {
+			if (PASS == xmysqlnd_crud_table_delete__set_criteria(data_object.crud_op, where_expr_str)) {
 				ZVAL_COPY(return_value, object_zv);
 			}
 		}
@@ -126,13 +115,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, where)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, orderby)
 {
-	st_mysqlx_table__delete* object{nullptr};
+	DBG_ENTER("mysqlx_table__delete::orderby");
+
 	zval* object_zv{nullptr};
 	zval* orderby_expr{nullptr};
 	int num_of_expr{0};
-
-	DBG_ENTER("mysqlx_table__delete::orderby");
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O+",
 									&object_zv,
 									mysqlx_table__delete_class_entry,
@@ -142,11 +129,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, orderby)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (!( object->crud_op && orderby_expr ) ) {
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__delete>(object_zv) };
+	if (!( data_object.crud_op && orderby_expr ) ) {
 		DBG_VOID_RETURN;
 	}
 
@@ -157,7 +143,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, orderby)
 			{
 				const MYSQLND_CSTRING orderby_expr_str = {Z_STRVAL(orderby_expr[i]),
 													Z_STRLEN(orderby_expr[i])};
-				if (PASS == xmysqlnd_crud_table_delete__add_orderby(object->crud_op, orderby_expr_str)) {
+				if (PASS == xmysqlnd_crud_table_delete__add_orderby(data_object.crud_op, orderby_expr_str)) {
 					ZVAL_COPY(return_value, object_zv);
 				}
 			}
@@ -172,7 +158,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, orderby)
 						RAISE_EXCEPTION(err_msg_wrong_param_1);
 						DBG_VOID_RETURN;
 					}
-					if (FAIL == xmysqlnd_crud_table_delete__add_orderby(object->crud_op, orderby_expr_str))
+					if (FAIL == xmysqlnd_crud_table_delete__add_orderby(data_object.crud_op, orderby_expr_str))
 					{
 						RAISE_EXCEPTION(err_msg_add_orderby_fail);
 						DBG_VOID_RETURN;
@@ -191,12 +177,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, orderby)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, limit)
 {
-	st_mysqlx_table__delete* object{nullptr};
-	zval* object_zv{nullptr};
-	zend_long rows;
-
 	DBG_ENTER("mysqlx_table__delete::limit");
 
+	zval* object_zv{nullptr};
+	zend_long rows;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Ol",
 		&object_zv, mysqlx_table__delete_class_entry,
 		&rows))
@@ -204,19 +188,16 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, limit)
 		DBG_VOID_RETURN;
 	}
 
-	if (rows < 0)
-	{
+	if (rows < 0) {
 		RAISE_EXCEPTION(err_msg_wrong_param_2);
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->crud_op)
-	{
-		if (PASS == xmysqlnd_crud_table_delete__set_limit(object->crud_op, rows)) {
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__delete>(object_zv) };
+	if (data_object.crud_op) {
+		if (PASS == xmysqlnd_crud_table_delete__set_limit(data_object.crud_op, rows)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
@@ -226,12 +207,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, limit)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, bind)
 {
-	st_mysqlx_table__delete* object{nullptr};
-	zval* object_zv{nullptr};
-	HashTable * bind_variables;
-
 	DBG_ENTER("mysqlx_table__delete::bind");
 
+	zval* object_zv{nullptr};
+	HashTable * bind_variables;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Oh",
 		&object_zv, mysqlx_table__delete_class_entry,
 		&bind_variables))
@@ -239,20 +218,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, bind)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	if (object->crud_op)
-	{
-		zend_string * key;
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__delete>(object_zv) };
+	if (data_object.crud_op) {
+		zend_string* key;
 		zval* val{nullptr};
 		MYSQLX_HASH_FOREACH_STR_KEY_VAL(bind_variables, key, val)
 		{
 			if (key)
 			{
 				const MYSQLND_CSTRING variable = {ZSTR_VAL(key), ZSTR_LEN(key)};
-				if (FAIL == xmysqlnd_crud_table_delete__bind_value(object->crud_op, variable, val))
+				if (FAIL == xmysqlnd_crud_table_delete__bind_value(data_object.crud_op, variable, val))
 				{
 					RAISE_EXCEPTION(err_msg_bind_fail);
 					DBG_VOID_RETURN;
@@ -266,27 +243,24 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, bind)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__delete, execute)
 {
-	st_mysqlx_table__delete* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table__delete::execute");
 
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table__delete_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
-
 	RETVAL_FALSE;
 
-	DBG_INF_FMT("crud_op=%p table=%p", object->crud_op, object->table);
-	if (object->crud_op && object->table) {
-		if (FALSE == xmysqlnd_crud_table_delete__is_initialized(object->crud_op)) {
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__delete>(object_zv) };
+	DBG_INF_FMT("crud_op=%p table=%p", data_object.crud_op, data_object.table);
+	if (data_object.crud_op && data_object.table) {
+		if (FALSE == xmysqlnd_crud_table_delete__is_initialized(data_object.crud_op)) {
 			RAISE_EXCEPTION(err_msg_delete_fail);
 		} else {
-			xmysqlnd_stmt * stmt = object->table->opdelete( object->crud_op);
+			xmysqlnd_stmt* stmt = data_object.table->opdelete(data_object.crud_op);
 			if (stmt) {
 				zval stmt_zv;
 				ZVAL_UNDEF(&stmt_zv);
@@ -325,7 +299,7 @@ static const zend_function_entry mysqlx_table__delete_methods[] = {
 
 #if 0
 static zval *
-mysqlx_table__delete_property__name(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_table__delete_property__name(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_table__delete* object = (const st_mysqlx_table__delete* ) (obj->ptr);
 	DBG_ENTER("mysqlx_table__delete_property__name");
@@ -360,25 +334,11 @@ const struct st_mysqlx_property_entry mysqlx_table__delete_property_entries[] =
 static void
 mysqlx_table__delete_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_table__delete* inner_obj = (st_mysqlx_table__delete*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->table) {
-			xmysqlnd_table_free(inner_obj->table, nullptr, nullptr);
-			inner_obj->table = nullptr;
-		}
-		if(inner_obj->crud_op) {
-			xmysqlnd_crud_table_delete__destroy(inner_obj->crud_op);
-			inner_obj->crud_op = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_table__delete>(object);
 }
 
 static zend_object *
-php_mysqlx_table__delete_object_allocator(zend_class_entry * class_type)
+php_mysqlx_table__delete_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_table__delete_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_table__delete>(
@@ -389,23 +349,20 @@ php_mysqlx_table__delete_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_table__delete_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_table__delete_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_table__delete_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_table__delete_handlers.free_obj = mysqlx_table__delete_free_storage;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_table__delete_class_entry,
+		"TableDelete",
+		mysqlx_std_object_handlers,
+		mysqlx_object_table__delete_handlers,
+		php_mysqlx_table__delete_object_allocator,
+		mysqlx_table__delete_free_storage,
+		mysqlx_table__delete_methods,
+		mysqlx_table__delete_properties,
+		mysqlx_table__delete_property_entries,
+		mysqlx_executable_interface_entry);
 
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "TableDelete", mysqlx_table__delete_methods);
-		tmp_ce.create_object = php_mysqlx_table__delete_object_allocator;
-		mysqlx_table__delete_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_table__delete_class_entry, 1, mysqlx_executable_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_table__delete_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_table__delete_properties, mysqlx_table__delete_property_entries);
 #if 0
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_table__delete_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -419,25 +376,15 @@ mysqlx_unregister_table__delete_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_table__delete(zval * return_value, xmysqlnd_table * table, const zend_bool clone)
+mysqlx_new_table__delete(zval* return_value, xmysqlnd_table* table)
 {
 	DBG_ENTER("mysqlx_new_table__delete");
-
-	if (SUCCESS == object_init_ex(return_value, mysqlx_table__delete_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_table__delete* const object = (st_mysqlx_table__delete*) mysqlx_object->ptr;
-		if (object) {
-			object->table = clone? table->get_reference(): table;
-			object->crud_op = xmysqlnd_crud_table_delete__create(
-				mnd_str2c(object->table->get_schema()->get_name()),
-				mnd_str2c(object->table->get_name()));
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
-
+	st_mysqlx_table__delete& data_object{
+		util::init_object<st_mysqlx_table__delete>(mysqlx_table__delete_class_entry, return_value) };
+	data_object.table = table->get_reference();
+	data_object.crud_op = xmysqlnd_crud_table_delete__create(
+		mnd_str2c(data_object.table->get_schema()->get_name()),
+		mnd_str2c(data_object.table->get_name()));
 	DBG_VOID_RETURN;
 }
 
