@@ -52,20 +52,16 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_table__insert : public util::custom_allocable
 {
-	XMYSQLND_CRUD_TABLE_OP__INSERT * crud_op;
-	xmysqlnd_table * table;
+	~st_mysqlx_table__insert();
+	XMYSQLND_CRUD_TABLE_OP__INSERT* crud_op;
+	xmysqlnd_table* table;
 };
 
-
-#define MYSQLX_FETCH_TABLE_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_table__insert*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->table) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
+st_mysqlx_table__insert::~st_mysqlx_table__insert()
+{
+	xmysqlnd_table_free(table, nullptr, nullptr);
+	xmysqlnd_crud_table_insert__destroy(crud_op);
+}
 
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, __construct)
@@ -75,14 +71,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, values)
 {
-	st_mysqlx_table__insert* object{nullptr};
+	DBG_ENTER("mysqlx_table__insert::values");
+
 	zval* object_zv{nullptr};
 	zval* values{nullptr};
 	zend_bool op_failed{FALSE};
 	int num_of_values{0};
-
-	DBG_ENTER("mysqlx_table__insert::values");
-
 	if (FAILURE == util::zend::parse_method_parameters(
 		execute_data, getThis(), "O+",
 		&object_zv,
@@ -93,7 +87,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, values)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__insert>(object_zv) };
 
 	RETVAL_FALSE;
 
@@ -101,7 +95,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, values)
 				num_of_values);
 
 	for(int i{0}; i < num_of_values ; ++i ) {
-		if (FAIL == xmysqlnd_crud_table_insert__add_row(object->crud_op,
+		if (FAIL == xmysqlnd_crud_table_insert__add_row(data_object.crud_op,
 												&values[i])) {
 			op_failed = TRUE;
 			break;
@@ -117,27 +111,25 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, values)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__insert, execute)
 {
-	st_mysqlx_table__insert* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table__insert::execute");
 
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table__insert_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__insert>(object_zv) };
 
 	RETVAL_FALSE;
 
-	DBG_INF_FMT("crud_op=%p table=%p", object->crud_op, object->table);
-	if (object->crud_op && object->table) {
-		if (FALSE == xmysqlnd_crud_table_insert__is_initialized(object->crud_op)) {
+	DBG_INF_FMT("crud_op=%p table=%p", data_object.crud_op, data_object.table);
+	if (data_object.crud_op && data_object.table) {
+		if (FALSE == xmysqlnd_crud_table_insert__is_initialized(data_object.crud_op)) {
 			RAISE_EXCEPTION(err_msg_insert_fail);
 		} else {
-			xmysqlnd_stmt * stmt = object->table->insert(object->crud_op);
+			xmysqlnd_stmt* stmt = data_object.table->insert(data_object.crud_op);
 			if (stmt) {
 				zval stmt_zv;
 				ZVAL_UNDEF(&stmt_zv);
@@ -173,7 +165,7 @@ static const zend_function_entry mysqlx_table__insert_methods[] = {
 
 #if 0
 static zval *
-mysqlx_table__insert_property__name(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_table__insert_property__name(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_table__insert* object = (const st_mysqlx_table__insert* ) (obj->ptr);
 	DBG_ENTER("mysqlx_table__insert_property__name");
@@ -208,25 +200,11 @@ const struct st_mysqlx_property_entry mysqlx_table__insert_property_entries[] =
 static void
 mysqlx_table__insert_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_table__insert* inner_obj = (st_mysqlx_table__insert*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->table) {
-			xmysqlnd_table_free(inner_obj->table, nullptr, nullptr);
-			inner_obj->table = nullptr;
-		}
-		if(inner_obj->crud_op) {
-			xmysqlnd_crud_table_insert__destroy(inner_obj->crud_op);
-			inner_obj->crud_op = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_table__insert>(object);
 }
 
 static zend_object *
-php_mysqlx_table__insert_object_allocator(zend_class_entry * class_type)
+php_mysqlx_table__insert_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_table__insert_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_table__insert>(
@@ -237,23 +215,20 @@ php_mysqlx_table__insert_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_table__insert_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_table__insert_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_table__insert_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_table__insert_handlers.free_obj = mysqlx_table__insert_free_storage;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_table__insert_class_entry,
+		"TableInsert",
+		mysqlx_std_object_handlers,
+		mysqlx_object_table__insert_handlers,
+		php_mysqlx_table__insert_object_allocator,
+		mysqlx_table__insert_free_storage,
+		mysqlx_table__insert_methods,
+		mysqlx_table__insert_properties,
+		mysqlx_table__insert_property_entries,
+		mysqlx_executable_interface_entry);
 
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "TableInsert", mysqlx_table__insert_methods);
-		tmp_ce.create_object = php_mysqlx_table__insert_object_allocator;
-		mysqlx_table__insert_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_table__insert_class_entry, 1, mysqlx_executable_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_table__insert_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_table__insert_properties, mysqlx_table__insert_property_entries);
 #if 0
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_table__insert_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -267,31 +242,21 @@ mysqlx_unregister_table__insert_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_table__insert(zval * return_value,
-					xmysqlnd_table * table,
-					const zend_bool clone,
-					zval * columns,
-					const int num_of_columns)
+mysqlx_new_table__insert(
+	zval* return_value,
+	xmysqlnd_table* table,
+	zval* columns,
+	const int num_of_columns)
 {
 	DBG_ENTER("mysqlx_new_table__insert");
-
-	if (SUCCESS == object_init_ex(return_value, mysqlx_table__insert_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_table__insert* const object = (st_mysqlx_table__insert*) mysqlx_object->ptr;
-		if (object) {
-			object->table = clone? table->get_reference() : table;
-			object->crud_op = xmysqlnd_crud_table_insert__create(
-				mnd_str2c(object->table->get_schema()->get_name()),
-				mnd_str2c(object->table->get_name()),
-				columns,
-				num_of_columns);
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
-
+	st_mysqlx_table__insert& data_object{
+		util::init_object<st_mysqlx_table__insert>(mysqlx_table__insert_class_entry, return_value) };
+	data_object.table = table->get_reference();
+	data_object.crud_op = xmysqlnd_crud_table_insert__create(
+		mnd_str2c(data_object.table->get_schema()->get_name()),
+		mnd_str2c(data_object.table->get_name()),
+		columns,
+		num_of_columns);
 	DBG_VOID_RETURN;
 }
 
