@@ -90,20 +90,16 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_table__select : public util::custom_allocable
 {
-	XMYSQLND_CRUD_TABLE_OP__SELECT * crud_op;
-	xmysqlnd_table * table;
+	~st_mysqlx_table__select();
+	XMYSQLND_CRUD_TABLE_OP__SELECT* crud_op;
+	xmysqlnd_table* table;
 };
 
-
-#define MYSQLX_FETCH_TABLE_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_table__select*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->table) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
+st_mysqlx_table__select::~st_mysqlx_table__select()
+{
+	xmysqlnd_table_free(table, nullptr, nullptr);
+	xmysqlnd_crud_table_select__destroy(crud_op);
+}
 
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, __construct)
@@ -113,12 +109,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, __construct)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, where)
 {
-	st_mysqlx_table__select* object{nullptr};
-	zval* object_zv{nullptr};
-	MYSQLND_CSTRING where_expr = {nullptr, 0};
-
 	DBG_ENTER("mysqlx_table__select::where");
 
+	zval* object_zv{nullptr};
+	MYSQLND_CSTRING where_expr = {nullptr, 0};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Os",
 												&object_zv, mysqlx_table__select_class_entry,
 												&(where_expr.s), &(where_expr.l)))
@@ -126,32 +120,28 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, where)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__select>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->table && where_expr.s && where_expr.l)
-	{
-		if (PASS == xmysqlnd_crud_table_select__set_criteria(object->crud_op, where_expr))
-		{
+	if (data_object.table && where_expr.s && where_expr.l) {
+		if (PASS == xmysqlnd_crud_table_select__set_criteria(data_object.crud_op, where_expr)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
 }
 
-#define ADD_SORT 1
-#define ADD_GROUPING 2
+const unsigned int ADD_SORT = 1;
+const unsigned int ADD_GROUPING = 2;
 
 static void
 mysqlx_table__select__add_sort_or_grouping(INTERNAL_FUNCTION_PARAMETERS, const unsigned int op_type)
 {
-	st_mysqlx_table__select* object{nullptr};
+	DBG_ENTER("mysqlx_table__select__add_sort_or_grouping");
+
 	zval* object_zv{nullptr};
 	zval* sort_expr{nullptr};
 	int num_of_expr{0};
-
-	DBG_ENTER("mysqlx_table__select__add_sort_or_grouping");
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O+",
 												&object_zv,
 												mysqlx_table__select_class_entry,
@@ -175,12 +165,11 @@ mysqlx_table__select__add_sort_or_grouping(INTERNAL_FUNCTION_PARAMETERS, const u
 		}
 	}
 
-
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__select>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (!( object->crud_op && sort_expr ) ) {
+	if (!( data_object.crud_op && sort_expr ) ) {
 		DBG_VOID_RETURN;
 	}
 
@@ -191,11 +180,11 @@ mysqlx_table__select__add_sort_or_grouping(INTERNAL_FUNCTION_PARAMETERS, const u
 			{
 				const MYSQLND_CSTRING sort_expr_str = { Z_STRVAL(sort_expr[i]), Z_STRLEN(sort_expr[i]) };
 				if (ADD_SORT == op_type) {
-					if (PASS == xmysqlnd_crud_table_select__add_orderby(object->crud_op, sort_expr_str)) {
+					if (PASS == xmysqlnd_crud_table_select__add_orderby(data_object.crud_op, sort_expr_str)) {
 						ZVAL_COPY(return_value, object_zv);
 					}
 				} else if (ADD_GROUPING == op_type) {
-					if (PASS == xmysqlnd_crud_table_select__add_grouping(object->crud_op, sort_expr_str)) {
+					if (PASS == xmysqlnd_crud_table_select__add_grouping(data_object.crud_op, sort_expr_str)) {
 						ZVAL_COPY(return_value, object_zv);
 					}
 				}
@@ -213,9 +202,9 @@ mysqlx_table__select__add_sort_or_grouping(INTERNAL_FUNCTION_PARAMETERS, const u
 						DBG_VOID_RETURN;
 					}
 					if (ADD_SORT == op_type) {
-						ret = xmysqlnd_crud_table_select__add_orderby(object->crud_op, sort_expr_str);
+						ret = xmysqlnd_crud_table_select__add_orderby(data_object.crud_op, sort_expr_str);
 					} else if (ADD_GROUPING == op_type) {
-						ret = xmysqlnd_crud_table_select__add_grouping(object->crud_op, sort_expr_str);
+						ret = xmysqlnd_crud_table_select__add_grouping(data_object.crud_op, sort_expr_str);
 					}
 					if (FAIL == ret) {
 						RAISE_EXCEPTION(err_msg_add_sort_fail);
@@ -251,12 +240,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, groupBy)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, having)
 {
-	st_mysqlx_table__select* object{nullptr};
-	zval* object_zv{nullptr};
-	MYSQLND_CSTRING search_condition = {nullptr, 0};
-
 	DBG_ENTER("mysqlx_table__select::having");
 
+	zval* object_zv{nullptr};
+	MYSQLND_CSTRING search_condition = {nullptr, 0};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Os",
 												&object_zv, mysqlx_table__select_class_entry,
 												&(search_condition.s), &(search_condition.l)))
@@ -264,12 +251,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, having)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__select>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op && object->table) {
-		if (PASS == xmysqlnd_crud_table_select__set_having(object->crud_op, search_condition)) {
+	if (data_object.crud_op && data_object.table) {
+		if (PASS == xmysqlnd_crud_table_select__set_having(data_object.crud_op, search_condition)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
@@ -279,12 +266,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, having)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, limit)
 {
-	st_mysqlx_table__select* object{nullptr};
-	zval* object_zv{nullptr};
-	zend_long rows;
-
 	DBG_ENTER("mysqlx_table__select::limit");
 
+	zval* object_zv{nullptr};
+	zend_long rows;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Ol",
 												&object_zv, mysqlx_table__select_class_entry,
 												&rows))
@@ -297,12 +282,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, limit)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__select>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op && object->table) {
-		if (PASS == xmysqlnd_crud_table_select__set_limit(object->crud_op, rows)) {
+	if (data_object.crud_op && data_object.table) {
+		if (PASS == xmysqlnd_crud_table_select__set_limit(data_object.crud_op, rows)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
@@ -312,12 +297,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, limit)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, offset)
 {
-	st_mysqlx_table__select* object{nullptr};
-	zval* object_zv{nullptr};
-	zend_long position;
-
 	DBG_ENTER("mysqlx_table__select::offset");
 
+	zval* object_zv{nullptr};
+	zend_long position;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Ol",
 												&object_zv, mysqlx_table__select_class_entry,
 												&position))
@@ -330,12 +313,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, offset)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__select>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op && object->table) {
-		if (PASS == xmysqlnd_crud_table_select__set_offset(object->crud_op, position)) {
+	if (data_object.crud_op && data_object.table) {
+		if (PASS == xmysqlnd_crud_table_select__set_offset(data_object.crud_op, position)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
@@ -345,12 +328,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, offset)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, bind)
 {
-	st_mysqlx_table__select* object{nullptr};
-	zval* object_zv{nullptr};
-	HashTable * bind_variables;
-
 	DBG_ENTER("mysqlx_table__select::bind");
 
+	zval* object_zv{nullptr};
+	HashTable * bind_variables;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Oh",
 												&object_zv, mysqlx_table__select_class_entry,
 												&bind_variables))
@@ -358,17 +339,17 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__select, bind)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__select>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op && object->table) {
+	if (data_object.crud_op && data_object.table) {
 		zend_string * key;
 		zval* val{nullptr};
 		MYSQLX_HASH_FOREACH_STR_KEY_VAL(bind_variables, key, val) {
 			if (key) {
 				const MYSQLND_CSTRING variable = { ZSTR_VAL(key), ZSTR_LEN(key) };
-				if (FAIL == xmysqlnd_crud_table_select__bind_value(object->crud_op, variable, val)) {
+				if (FAIL == xmysqlnd_crud_table_select__bind_value(data_object.crud_op, variable, val)) {
 					RAISE_EXCEPTION(err_msg_bind_fail);
 					DBG_VOID_RETURN;
 				}
@@ -493,7 +474,7 @@ static const zend_function_entry mysqlx_table__select_methods[] = {
 
 #if 0
 static zval *
-mysqlx_table__select_property__name(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_table__select_property__name(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_table__select* object = (const st_mysqlx_table__select* ) (obj->ptr);
 	DBG_ENTER("mysqlx_table__select_property__name");
@@ -528,25 +509,11 @@ const struct st_mysqlx_property_entry mysqlx_table__select_property_entries[] =
 static void
 mysqlx_table__select_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_table__select* inner_obj = (st_mysqlx_table__select*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->table) {
-			xmysqlnd_table_free(inner_obj->table, nullptr, nullptr);
-			inner_obj->table = nullptr;
-		}
-		if(inner_obj->crud_op) {
-			xmysqlnd_crud_table_select__destroy(inner_obj->crud_op);
-			inner_obj->crud_op = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_table__select>(object);
 }
 
 static zend_object *
-php_mysqlx_table__select_object_allocator(zend_class_entry * class_type)
+php_mysqlx_table__select_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_table__select_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_table__select>(
@@ -557,23 +524,20 @@ php_mysqlx_table__select_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_table__select_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_table__select_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_table__select_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_table__select_handlers.free_obj = mysqlx_table__select_free_storage;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_table__select_class_entry,
+		"TableSelect",
+		mysqlx_std_object_handlers,
+		mysqlx_object_table__select_handlers,
+		php_mysqlx_table__select_object_allocator,
+		mysqlx_table__select_free_storage,
+		mysqlx_table__select_methods,
+		mysqlx_table__select_properties,
+		mysqlx_table__select_property_entries,
+		mysqlx_executable_interface_entry);
 
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "TableSelect", mysqlx_table__select_methods);
-		tmp_ce.create_object = php_mysqlx_table__select_object_allocator;
-		mysqlx_table__select_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_table__select_class_entry, 1, mysqlx_executable_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_table__select_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_table__select_properties, mysqlx_table__select_property_entries);
 #if 0
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_table__select_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -587,31 +551,21 @@ mysqlx_unregister_table__select_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_table__select(zval * return_value,
-				xmysqlnd_table * table,
-				const zend_bool clone,
-				zval * columns,
-				const int num_of_columns)
+mysqlx_new_table__select(
+	zval* return_value,
+	xmysqlnd_table* table,
+	zval* columns,
+	const int num_of_columns)
 {
 	DBG_ENTER("mysqlx_new_table__select");
-
-	if (SUCCESS == object_init_ex(return_value, mysqlx_table__select_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_table__select* const object = (st_mysqlx_table__select*) mysqlx_object->ptr;
-		if (object) {
-			object->table = clone? table->get_reference() : table;
-			object->crud_op = xmysqlnd_crud_table_select__create(
-				mnd_str2c(object->table->get_schema()->get_name()),
-				mnd_str2c(object->table->get_name()),
-				columns,
-				num_of_columns);
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
-
+	st_mysqlx_table__select& data_object{
+		util::init_object<st_mysqlx_table__select>(mysqlx_table__select_class_entry, return_value) };
+	data_object.table = table->get_reference();
+	data_object.crud_op = xmysqlnd_crud_table_select__create(
+		mnd_str2c(data_object.table->get_schema()->get_name()),
+		mnd_str2c(data_object.table->get_name()),
+		columns,
+		num_of_columns);
 	DBG_VOID_RETURN;
 }
 
