@@ -72,20 +72,16 @@ ZEND_END_ARG_INFO()
 
 struct st_mysqlx_table__update : public util::custom_allocable
 {
-	XMYSQLND_CRUD_TABLE_OP__UPDATE * crud_op;
-	xmysqlnd_table * table;
+	~st_mysqlx_table__update();
+	XMYSQLND_CRUD_TABLE_OP__UPDATE* crud_op;
+	xmysqlnd_table* table;
 };
 
-
-#define MYSQLX_FETCH_TABLE_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_table__update*) mysqlx_object->ptr; \
-	if (!(_to) || !(_to)->table) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		DBG_VOID_RETURN; \
-	} \
-} \
+st_mysqlx_table__update::~st_mysqlx_table__update()
+{
+	xmysqlnd_table_free(table, nullptr, nullptr);
+	xmysqlnd_crud_table_update__destroy(crud_op);
+}
 
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, __construct)
@@ -100,15 +96,13 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, __construct)
 static void
 mysqlx_table__update__2_param_op(INTERNAL_FUNCTION_PARAMETERS, const unsigned int op_type)
 {
-	st_mysqlx_table__update* object{nullptr};
+	DBG_ENTER("mysqlx_table__update__2_param_op");
+
 	zval* object_zv{nullptr};
 	zval* value{nullptr};
 	MYSQLND_CSTRING table_field = {nullptr, 0};
 	zend_bool is_expression{FALSE};
 	const zend_bool is_document = FALSE;
-
-	DBG_ENTER("mysqlx_table__update__2_param_op");
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Osz",
 												&object_zv, mysqlx_table__update_class_entry,
 											 	&(table_field.s), &(table_field.l),
@@ -140,15 +134,16 @@ mysqlx_table__update__2_param_op(INTERNAL_FUNCTION_PARAMETERS, const unsigned in
 		}
 
 	}
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__update>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op) {
+	if (data_object.crud_op) {
 		enum_func_status ret{FAIL};
 		switch (op_type) {
 			case TWO_PARAM_OP__SET:
-				ret = xmysqlnd_crud_table_update__set(object->crud_op, table_field, value, is_expression, is_document);
+				ret = xmysqlnd_crud_table_update__set(data_object.crud_op, table_field, value, is_expression, is_document);
 				break;
 		}
 
@@ -166,12 +161,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, set)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, where)
 {
-	st_mysqlx_table__update* object{nullptr};
-	zval* object_zv{nullptr};
-	MYSQLND_CSTRING where_expr = {nullptr, 0};
-
 	DBG_ENTER("mysqlx_table__update::where");
 
+	zval* object_zv{nullptr};
+	MYSQLND_CSTRING where_expr = {nullptr, 0};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Os",
 												&object_zv, mysqlx_table__update_class_entry,
 												&(where_expr.s), &(where_expr.l)))
@@ -179,14 +172,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, where)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__update>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op && where_expr.s && where_expr.l)
-	{
-		if (PASS == xmysqlnd_crud_table_update__set_criteria(object->crud_op, where_expr))
-		{
+	if (data_object.crud_op && where_expr.s && where_expr.l) {
+		if (PASS == xmysqlnd_crud_table_update__set_criteria(data_object.crud_op, where_expr)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
@@ -196,13 +187,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, where)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, orderby)
 {
-	st_mysqlx_table__update* object{nullptr};
+	DBG_ENTER("mysqlx_table__update::orderby");
+
 	zval* object_zv{nullptr};
 	zval* orderby_expr{nullptr};
 	int num_of_expr{0};
-
-	DBG_ENTER("mysqlx_table__update::orderby");
-
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O+",
 												&object_zv,
 												mysqlx_table__update_class_entry,
@@ -212,11 +201,11 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, orderby)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__update>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (!( object->crud_op && orderby_expr ) ) {
+	if (!( data_object.crud_op && orderby_expr ) ) {
 		DBG_VOID_RETURN;
 	}
 
@@ -226,7 +215,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, orderby)
 			{
 				const MYSQLND_CSTRING orderby_expr_str = { Z_STRVAL(orderby_expr[i]),
 												Z_STRLEN(orderby_expr[i]) };
-				if (PASS == xmysqlnd_crud_table_update__add_orderby(object->crud_op, orderby_expr_str)) {
+				if (PASS == xmysqlnd_crud_table_update__add_orderby(data_object.crud_op, orderby_expr_str)) {
 					ZVAL_COPY(return_value, object_zv);
 				}
 			}
@@ -240,7 +229,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, orderby)
 						RAISE_EXCEPTION(err_msg_wrong_param_1);
 						DBG_VOID_RETURN;
 					}
-					if (FAIL == xmysqlnd_crud_table_update__add_orderby(object->crud_op, orderby_expr_str)) {
+					if (FAIL == xmysqlnd_crud_table_update__add_orderby(data_object.crud_op, orderby_expr_str)) {
 						RAISE_EXCEPTION(err_msg_add_orderby_fail);
 						DBG_VOID_RETURN;
 					}
@@ -258,12 +247,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, orderby)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, limit)
 {
-	st_mysqlx_table__update* object{nullptr};
-	zval* object_zv{nullptr};
-	zend_long rows;
-
 	DBG_ENTER("mysqlx_table__update::limit");
 
+	zval* object_zv{nullptr};
+	zend_long rows;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Ol",
 												&object_zv, mysqlx_table__update_class_entry,
 												&rows))
@@ -276,12 +263,12 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, limit)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__update>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op) {
-		if (PASS == xmysqlnd_crud_table_update__set_limit(object->crud_op, rows)) {
+	if (data_object.crud_op) {
+		if (PASS == xmysqlnd_crud_table_update__set_limit(data_object.crud_op, rows)) {
 			ZVAL_COPY(return_value, object_zv);
 		}
 	}
@@ -291,12 +278,10 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, limit)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, bind)
 {
-	st_mysqlx_table__update* object{nullptr};
-	zval* object_zv{nullptr};
-	HashTable * bind_variables;
-
 	DBG_ENTER("mysqlx_table__update::bind");
 
+	zval* object_zv{nullptr};
+	HashTable * bind_variables;
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "Oh",
 												&object_zv, mysqlx_table__update_class_entry,
 												&bind_variables))
@@ -304,18 +289,18 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, bind)
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__update>(object_zv) };
 
 	RETVAL_FALSE;
 
-	if (object->crud_op) {
+	if (data_object.crud_op) {
 		zend_string * key;
 		zval* val{nullptr};
 		zend_bool op_success{TRUE};
 		MYSQLX_HASH_FOREACH_STR_KEY_VAL(bind_variables, key, val) {
 			if (key) {
 				const MYSQLND_CSTRING variable = { ZSTR_VAL(key), ZSTR_LEN(key) };
-				if (FAIL == xmysqlnd_crud_table_update__bind_value(object->crud_op, variable, val)) {
+				if (FAIL == xmysqlnd_crud_table_update__bind_value(data_object.crud_op, variable, val)) {
 					RAISE_EXCEPTION(err_msg_bind_fail);
 					op_success = FALSE;
 					break;
@@ -331,27 +316,25 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, bind)
 
 MYSQL_XDEVAPI_PHP_METHOD(mysqlx_table__update, execute)
 {
-	st_mysqlx_table__update* object{nullptr};
-	zval* object_zv{nullptr};
-
 	DBG_ENTER("mysqlx_table__update::execute");
 
+	zval* object_zv{nullptr};
 	if (FAILURE == util::zend::parse_method_parameters(execute_data, getThis(), "O",
 												&object_zv, mysqlx_table__update_class_entry))
 	{
 		DBG_VOID_RETURN;
 	}
 
-	MYSQLX_FETCH_TABLE_FROM_ZVAL(object, object_zv);
+	auto& data_object{ util::fetch_data_object<st_mysqlx_table__update>(object_zv) };
 
 	RETVAL_FALSE;
 
-	DBG_INF_FMT("crud_op=%p table=%p", object->crud_op, object->table);
-	if (object->crud_op && object->table) {
-		if (FALSE == xmysqlnd_crud_table_update__is_initialized(object->crud_op)) {
+	DBG_INF_FMT("crud_op=%p table=%p", data_object.crud_op, data_object.table);
+	if (data_object.crud_op && data_object.table) {
+		if (FALSE == xmysqlnd_crud_table_update__is_initialized(data_object.crud_op)) {
 			RAISE_EXCEPTION(err_msg_update_fail);
 		} else {
-			xmysqlnd_stmt * stmt = object->table->update(object->crud_op);
+			xmysqlnd_stmt * stmt = data_object.table->update(data_object.crud_op);
 			if (stmt) {
 				zval stmt_zv;
 				ZVAL_UNDEF(&stmt_zv);
@@ -392,7 +375,7 @@ static const zend_function_entry mysqlx_table__update_methods[] = {
 
 #if 0
 static zval *
-mysqlx_table__update_property__name(const st_mysqlx_object* obj, zval * return_value)
+mysqlx_table__update_property__name(const st_mysqlx_object* obj, zval* return_value)
 {
 	const st_mysqlx_table__update* object = (const st_mysqlx_table__update* ) (obj->ptr);
 	DBG_ENTER("mysqlx_table__update_property__name");
@@ -427,25 +410,11 @@ const struct st_mysqlx_property_entry mysqlx_table__update_property_entries[] =
 static void
 mysqlx_table__update_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_table__update* inner_obj = (st_mysqlx_table__update*) mysqlx_object->ptr;
-
-	if (inner_obj) {
-		if (inner_obj->table) {
-			xmysqlnd_table_free(inner_obj->table, nullptr, nullptr);
-			inner_obj->table = nullptr;
-		}
-		if(inner_obj->crud_op) {
-			xmysqlnd_crud_table_update__destroy(inner_obj->crud_op);
-			inner_obj->crud_op = nullptr;
-		}
-		mnd_efree(inner_obj);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_table__update>(object);
 }
 
 static zend_object *
-php_mysqlx_table__update_object_allocator(zend_class_entry * class_type)
+php_mysqlx_table__update_object_allocator(zend_class_entry* class_type)
 {
 	DBG_ENTER("php_mysqlx_table__update_object_allocator");
 	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_table__update>(
@@ -456,23 +425,20 @@ php_mysqlx_table__update_object_allocator(zend_class_entry * class_type)
 }
 
 void
-mysqlx_register_table__update_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_table__update_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_table__update_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_table__update_handlers.free_obj = mysqlx_table__update_free_storage;
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_table__update_class_entry,
+		"TableUpdate",
+		mysqlx_std_object_handlers,
+		mysqlx_object_table__update_handlers,
+		php_mysqlx_table__update_object_allocator,
+		mysqlx_table__update_free_storage,
+		mysqlx_table__update_methods,
+		mysqlx_table__update_properties,
+		mysqlx_table__update_property_entries,
+		mysqlx_executable_interface_entry);
 
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "TableUpdate", mysqlx_table__update_methods);
-		tmp_ce.create_object = php_mysqlx_table__update_object_allocator;
-		mysqlx_table__update_class_entry = zend_register_internal_class(&tmp_ce);
-		zend_class_implements(mysqlx_table__update_class_entry, 1, mysqlx_executable_interface_entry);
-	}
-
-	zend_hash_init(&mysqlx_table__update_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	/* Add name + getter + setter to the hash table with the properties for the class */
-	mysqlx_add_properties(&mysqlx_table__update_properties, mysqlx_table__update_property_entries);
 #if 0
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_table__update_class_entry, "name",	sizeof("name") - 1,	ZEND_ACC_PUBLIC);
@@ -486,25 +452,15 @@ mysqlx_unregister_table__update_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_table__update(zval * return_value, xmysqlnd_table * table, const zend_bool clone)
+mysqlx_new_table__update(zval* return_value, xmysqlnd_table* table)
 {
 	DBG_ENTER("mysqlx_new_table__update");
-
-	if (SUCCESS == object_init_ex(return_value, mysqlx_table__update_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_table__update* const object = (st_mysqlx_table__update*) mysqlx_object->ptr;
-		if (object) {
-			object->table = clone? table->get_reference() : table;
-			object->crud_op = xmysqlnd_crud_table_update__create(
-				mnd_str2c(object->table->get_schema()->get_name()),
-				mnd_str2c(object->table->get_name()));
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
-
+	st_mysqlx_table__update& data_object{
+		util::init_object<st_mysqlx_table__update>(mysqlx_table__update_class_entry, return_value) };
+	data_object.table = table->get_reference();
+	data_object.crud_op = xmysqlnd_crud_table_update__create(
+		mnd_str2c(data_object.table->get_schema()->get_name()),
+		mnd_str2c(data_object.table->get_name()));
 	DBG_VOID_RETURN;
 }
 
