@@ -31,24 +31,18 @@ using namespace drv;
 
 static zend_class_entry * mysqlx_warning_class_entry;
 
-struct st_mysqlx_warning
+struct st_mysqlx_warning : public util::custom_allocable
 {
+	~st_mysqlx_warning();
 	MYSQLND_STRING msg;
 	unsigned int level;
 	unsigned int code;
 	zend_bool persistent;
 };
 
-
-#define MYSQLX_FETCH_WARNING_FROM_ZVAL(_to, _from) \
-{ \
-	const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P((_from)); \
-	(_to) = (st_mysqlx_warning*) mysqlx_object->ptr; \
-	if (!(_to)) { \
-		php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name)); \
-		RETVAL_NULL(); \
-		DBG_VOID_RETURN; \
-	} \
+st_mysqlx_warning::~st_mysqlx_warning()
+{
+	mnd_efree(msg.s);
 }
 
 
@@ -62,13 +56,13 @@ static const zend_function_entry mysqlx_warning_methods[] = {
 	{nullptr, nullptr, nullptr}
 };
 
-static zval *
-mysqlx_warning_property__message(const st_mysqlx_object* obj, zval * return_value)
+static zval*
+mysqlx_warning_property__message(const st_mysqlx_object* obj, zval* return_value)
 {
-	const st_mysqlx_warning* object = (const st_mysqlx_warning* ) (obj->ptr);
 	DBG_ENTER("mysqlx_warning_property__message");
-	if (object->msg.s) {
-		ZVAL_STRINGL(return_value, object->msg.s, object->msg.l);
+	const st_mysqlx_warning& data_object{ util::fetch_data_object<st_mysqlx_warning>(obj) };
+	if (data_object.msg.s) {
+		ZVAL_STRINGL(return_value, data_object.msg.s, data_object.msg.l);
 	} else {
 		/*
 		  This means EG(uninitialized_value). If we return just return_value, this is an UNDEF-ed value
@@ -82,22 +76,22 @@ mysqlx_warning_property__message(const st_mysqlx_object* obj, zval * return_valu
 	DBG_RETURN(return_value);
 }
 
-static zval *
-mysqlx_warning_property__level(const st_mysqlx_object* obj, zval * return_value)
+static zval*
+mysqlx_warning_property__level(const st_mysqlx_object* obj, zval* return_value)
 {
-	const st_mysqlx_warning* object = (const st_mysqlx_warning* ) (obj->ptr);
 	DBG_ENTER("mysqlx_warning_property__level");
-	ZVAL_LONG(return_value, object->level);
+	const st_mysqlx_warning& data_object{ util::fetch_data_object<st_mysqlx_warning>(obj) };
+	ZVAL_LONG(return_value, data_object.level);
 	DBG_RETURN(return_value);
 }
 
-static zval *
-mysqlx_warning_property__code(const st_mysqlx_object* obj, zval * return_value)
+static zval*
+mysqlx_warning_property__code(const st_mysqlx_object* obj, zval* return_value)
 {
-	const st_mysqlx_warning* object = (const st_mysqlx_warning* ) (obj->ptr);
 	DBG_ENTER("mysqlx_warning_property__code");
+	const st_mysqlx_warning& data_object{ util::fetch_data_object<st_mysqlx_warning>(obj) };
 	/* code is 32 bit unsigned and on 32bit system won't fit into 32 bit signed zend_long, but this won't happen in practice*/
-	ZVAL_LONG(return_value, object->code);
+	ZVAL_LONG(return_value, data_object.code);
 	DBG_RETURN(return_value);
 }
 
@@ -116,65 +110,33 @@ static HashTable mysqlx_warning_properties;
 static void
 mysqlx_warning_free_storage(zend_object * object)
 {
-	st_mysqlx_object* mysqlx_object = mysqlx_fetch_object_from_zo(object);
-	st_mysqlx_warning* message = (st_mysqlx_warning*) mysqlx_object->ptr;
-
-	if (message) {
-		if (message->msg.s) {
-			mnd_efree(message->msg.s);
-			message->msg.s = nullptr;
-		}
-		mnd_efree(message);
-	}
-	mysqlx_object_free_storage(object);
+	util::free_object<st_mysqlx_warning>(object);
 }
 
 static zend_object *
-php_mysqlx_warning_object_allocator(zend_class_entry * class_type)
+php_mysqlx_warning_object_allocator(zend_class_entry* class_type)
 {
-	const zend_bool persistent = FALSE;
-	st_mysqlx_object* mysqlx_object = (st_mysqlx_object*) mnd_ecalloc(1, sizeof(struct st_mysqlx_object) + zend_object_properties_size(class_type));
-	st_mysqlx_warning* message = (st_mysqlx_warning*)  mnd_ecalloc(1, sizeof(struct st_mysqlx_warning));
-
 	DBG_ENTER("php_mysqlx_warning_object_allocator");
-	if ( mysqlx_object && message) {
-		mysqlx_object->ptr = message;
-
-		message->persistent = persistent;
-		zend_object_std_init(&mysqlx_object->zo, class_type);
-		object_properties_init(&mysqlx_object->zo, class_type);
-
-		mysqlx_object->zo.handlers = &mysqlx_object_warning_handlers;
-		mysqlx_object->properties = &mysqlx_warning_properties;
-
-		DBG_RETURN(&mysqlx_object->zo);
-
-	}
-	if (message) {
-		mnd_efree(message);
-	}
-	if (mysqlx_object) {
-		mnd_efree(mysqlx_object);
-	}
-	DBG_RETURN(nullptr);
+	st_mysqlx_object* mysqlx_object = util::alloc_object<st_mysqlx_warning>(
+		class_type,
+		&mysqlx_object_warning_handlers,
+		&mysqlx_warning_properties);
+	DBG_RETURN(&mysqlx_object->zo);
 }
 
 void
-mysqlx_register_warning_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers * mysqlx_std_object_handlers)
+mysqlx_register_warning_class(UNUSED_INIT_FUNC_ARGS, zend_object_handlers* mysqlx_std_object_handlers)
 {
-	mysqlx_object_warning_handlers = *mysqlx_std_object_handlers;
-	mysqlx_object_warning_handlers.free_obj = mysqlx_warning_free_storage;
-
-	{
-		zend_class_entry tmp_ce;
-		INIT_NS_CLASS_ENTRY(tmp_ce, "mysql_xdevapi", "Warning", mysqlx_warning_methods);
-		tmp_ce.create_object = php_mysqlx_warning_object_allocator;
-		mysqlx_warning_class_entry = zend_register_internal_class(&tmp_ce);
-	}
-
-	zend_hash_init(&mysqlx_warning_properties, 0, nullptr, mysqlx_free_property_cb, 1);
-
-	mysqlx_add_properties(&mysqlx_warning_properties, mysqlx_warning_property_entries);
+	MYSQL_XDEVAPI_REGISTER_CLASS(
+		mysqlx_warning_class_entry,
+		"Warning",
+		mysqlx_std_object_handlers,
+		mysqlx_object_warning_handlers,
+		php_mysqlx_warning_object_allocator,
+		mysqlx_warning_free_storage,
+		mysqlx_warning_methods,
+		mysqlx_warning_properties,
+		mysqlx_warning_property_entries);
 
 	/* The following is needed for the Reflection API */
 	zend_declare_property_null(mysqlx_warning_class_entry, "message",	sizeof("message") - 1,		ZEND_ACC_PUBLIC);
@@ -189,22 +151,14 @@ mysqlx_unregister_warning_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 }
 
 void
-mysqlx_new_warning(zval * return_value, const MYSQLND_CSTRING msg, unsigned int level, const unsigned int code)
+mysqlx_new_warning(zval* return_value, const MYSQLND_CSTRING msg, unsigned int level, const unsigned int code)
 {
 	DBG_ENTER("mysqlx_new_warning");
-	if (SUCCESS == object_init_ex(return_value, mysqlx_warning_class_entry) && IS_OBJECT == Z_TYPE_P(return_value)) {
-		const st_mysqlx_object* const mysqlx_object = Z_MYSQLX_P(return_value);
-		st_mysqlx_warning* const object = (st_mysqlx_warning*) mysqlx_object->ptr;
-		if (object) {
-			object->msg = mnd_dup_cstring(msg, object->persistent);
-			object->level = level;
-			object->code = code;
-		} else {
-			php_error_docref(nullptr, E_WARNING, "invalid object of class %s", ZSTR_VAL(mysqlx_object->zo.ce->name));
-			zval_ptr_dtor(return_value);
-			ZVAL_NULL(return_value);
-		}
-	}
+	st_mysqlx_warning& data_object{
+		util::init_object<st_mysqlx_warning>(mysqlx_warning_class_entry, return_value) };
+	data_object.msg = mnd_dup_cstring(msg, data_object.persistent);
+	data_object.level = level;
+	data_object.code = code;
 	DBG_VOID_RETURN;
 }
 
