@@ -20,6 +20,7 @@
 #define MYSQL_XDEVAPI_UTIL_JSON_UTILS_H
 
 #include "strings.h"
+#include "xmysqlnd/proto_gen/mysqlx_datatypes.pb.h"
 #include <boost/version.hpp>
 #include <boost/property_tree/json_parser.hpp>
 
@@ -34,18 +35,20 @@ namespace json {
 void to_zv_string(zval* src, zval* dest);
 util::zvalue to_zv_string(const util::zvalue& src);
 
+util::zvalue to_zv_object(const char* src, const std::size_t src_len);
+
+
 bool can_be_document(const util::zvalue& value);
 bool can_be_array(const util::zvalue& value);
 bool can_be_binding(const util::zvalue& value);
 
-void ensure_doc_id(
+util::zvalue ensure_doc_id(
 	zval* raw_doc,
-	const string_view& id,
-	zval* doc_with_id);
+	const string_view& id);
 
-void ensure_doc_id_as_string(
-	const string_view& doc_id,
-	zval* doc);
+util::zvalue to_zval(const char* doc, std::size_t doc_len);
+
+void to_any(const char* doc, std::size_t doc_len, Mysqlx::Datatypes::Any& any);
 
 /*
 	in older versions of boost (e.g. 1.53.0 which is at the moment still officially
