@@ -13,6 +13,8 @@
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: Andrey Hristov <andrey@php.net>                             |
+  |          Filip Janiszewski <fjanisze@php.net>                        |
+  |          Darek Slusarczyk <marines@php.net>                          |
   +----------------------------------------------------------------------+
 */
 #include "php_api.h"
@@ -218,11 +220,8 @@ xmysqlnd_collection::add(XMYSQLND_CRUD_COLLECTION_OP__ADD * crud_op)
 	DBG_ENTER("xmysqlnd_collection::add");
 	xmysqlnd_stmt*                         ret{nullptr};
 	XMYSQLND_SESSION                       session{ get_schema()->get_session() };
-	struct st_xmysqlnd_message_factory     msg_factory;
 	if( xmysqlnd_crud_collection_add__finalize_bind(crud_op) == PASS ) {
-		msg_factory = xmysqlnd_get_message_factory(&session->data->io,
-											session->data->stats,
-											session->data->error_info);
+		st_xmysqlnd_message_factory msg_factory{ session->data->create_message_factory() };
 		st_xmysqlnd_msg__collection_add collection_add = msg_factory.get__collection_add(&msg_factory);
 		enum_func_status request_ret = collection_add.send_request(&collection_add,
 											xmysqlnd_crud_collection_add__get_protobuf_message(crud_op));
@@ -254,8 +253,7 @@ xmysqlnd_collection::remove(XMYSQLND_CRUD_COLLECTION_OP__REMOVE * op)
 			DBG_RETURN(stmt);
 		}
 		if (xmysqlnd_crud_collection_remove__is_initialized(op)) {
-			const struct st_xmysqlnd_message_factory msg_factory = xmysqlnd_get_message_factory(&session->data->io,
-																				session->data->stats, session->data->error_info);
+			st_xmysqlnd_message_factory msg_factory{ session->data->create_message_factory() };
 			struct st_xmysqlnd_msg__collection_ud collection_ud = msg_factory.get__collection_ud(&msg_factory);
 			if (PASS == collection_ud.send_delete_request(&collection_ud, xmysqlnd_crud_collection_remove__get_protobuf_message(op))) {
 				stmt = session->create_statement_object(session);
@@ -311,7 +309,7 @@ xmysqlnd_collection::modify(XMYSQLND_CRUD_COLLECTION_OP__MODIFY * op)
 			DBG_RETURN(stmt);
 		}
 		if (xmysqlnd_crud_collection_modify__is_initialized(op)) {
-			const struct st_xmysqlnd_message_factory msg_factory = xmysqlnd_get_message_factory(&session->data->io, session->data->stats, session->data->error_info);
+			st_xmysqlnd_message_factory msg_factory{ session->data->create_message_factory() };
 			struct st_xmysqlnd_msg__collection_ud collection_ud = msg_factory.get__collection_ud(&msg_factory);
 			if (PASS == collection_ud.send_update_request(&collection_ud, xmysqlnd_crud_collection_modify__get_protobuf_message(op))) {
 				stmt = session->create_statement_object(session);
