@@ -26,7 +26,7 @@ struct st_collection_op_var_binder_ctx
 {
 	const MYSQLND_CSTRING schema_name;
 	const MYSQLND_CSTRING collection_name;
-	unsigned int counter;
+	const MYSQLND_CSTRING collection_options;
 };
 
 } // namespace drv
